# Copyright 2016 Andrew Kerr
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import ddt
from tempest import config
from tempest.lib.common.utils import data_utils
from tempest.lib import decorators
from tempest.lib import exceptions as lib_exc
import testtools
from testtools import testcase as tc

from manila_tempest_tests.common import constants
from manila_tempest_tests.common import waiters
from manila_tempest_tests.tests.api import base
from manila_tempest_tests import utils

CONF = config.CONF


@ddt.ddt
class ShareGroupsTest(base.BaseSharesMixedTest):
    """Covers share group functionality."""

    @classmethod
    def skip_checks(cls):
        super(ShareGroupsTest, cls).skip_checks()
        if not CONF.share.run_share_group_tests:
            raise cls.skipException('Share Group tests disabled.')

        utils.check_skip_if_microversion_not_supported(
            constants.MIN_SHARE_GROUP_MICROVERSION)

    @classmethod
    def resource_setup(cls):
        super(ShareGroupsTest, cls).resource_setup()
        # create share  type
        extra_specs = {}
        if CONF.share.capability_snapshot_support:
            extra_specs.update({'snapshot_support': True})
        if CONF.share.capability_create_share_from_snapshot_support:
            extra_specs.update({'create_share_from_snapshot_support': True})
        cls.share_type = cls.create_share_type(extra_specs=extra_specs)
        cls.share_type_id = cls.share_type['id']

        # create share group type
        cls.share_group_type = cls._create_share_group_type()
        cls.share_group_type_id = cls.share_group_type['id']

    @decorators.idempotent_id('809d5e3d-5a4b-458a-a985-853d59800da5')
    @tc.attr(base.TAG_POSITIVE, base.TAG_API_WITH_BACKEND)
    def test_create_populate_delete_share_group_min(self):
        # Create a share group
        share_group = self.create_share_group(
            cleanup_in_class=False,
            version=constants.MIN_SHARE_GROUP_MICROVERSION,
            share_group_type_id=self.share_group_type_id,
            share_type_ids=[self.share_type_id],
        )

        keys = set(share_group.keys())
        self.assertTrue(
            constants.SHARE_GROUP_DETAIL_REQUIRED_KEYS.issubset(keys),
            'At least one expected element missing from share group '
            'response. Expected %(expected)s, got %(actual)s.' % {
                "expected": constants.SHARE_GROUP_DETAIL_REQUIRED_KEYS,
                "actual": keys}
        )
        # Populate
        share = self.create_share(
            share_type_id=self.share_type_id,
            share_group_id=share_group['id'],
            cleanup_in_class=False,
            version=constants.MIN_SHARE_GROUP_MICROVERSION)

        # Delete
        params = {"share_group_id": share_group['id']}
        self.shares_v2_client.delete_share(
            share['id'],
            params=params,
            version=constants.MIN_SHARE_GROUP_MICROVERSION)
        self.shares_client.wait_for_resource_deletion(share_id=share['id'])
        self.shares_v2_client.delete_share_group(
            share_group['id'], version=constants.MIN_SHARE_GROUP_MICROVERSION)
        self.shares_v2_client.wait_for_resource_deletion(
            share_group_id=share_group['id'])

        # Verify
        self.assertRaises(
            lib_exc.NotFound,
            self.shares_v2_client.get_share_group, share_group['id'])
        self.assertRaises(
            lib_exc.NotFound, self.shares_client.get_share, share['id'])

    @decorators.idempotent_id('cf7984af-1e1d-4eaf-bf9a-d8ddf5cebd01')
    @tc.attr(base.TAG_POSITIVE, base.TAG_API_WITH_BACKEND)
    @testtools.skipUnless(CONF.share.run_snapshot_tests,
                          "Snapshot tests are disabled.")
    def test_create_delete_empty_share_group_snapshot_min(self):
        # Create base share group
        share_group = self.create_share_group(
            share_group_type_id=self.share_group_type_id,
            share_type_ids=[self.share_type_id],
            cleanup_in_class=False,
            version=constants.MIN_SHARE_GROUP_MICROVERSION)

        # Create share group snapshot
        sg_snapshot = self.create_share_group_snapshot_wait_for_active(
            share_group["id"],
            cleanup_in_class=False,
            version=constants.MIN_SHARE_GROUP_MICROVERSION)

        keys = set(sg_snapshot.keys())
        self.assertTrue(
            constants.SHARE_GROUP_SNAPSHOT_DETAIL_REQUIRED_KEYS.issubset(keys),
            'At least one expected element missing from share group snapshot '
            'response. Expected %(e)s, got %(a)s.' % {
                "e": constants.SHARE_GROUP_SNAPSHOT_DETAIL_REQUIRED_KEYS,
                "a": keys})

        sg_snapshot_members = sg_snapshot['members']
        self.assertEmpty(
            sg_snapshot_members,
            'Expected 0 share_group_snapshot members, got %s' % len(
                sg_snapshot_members))

        # Delete snapshot
        self.shares_v2_client.delete_share_group_snapshot(
            sg_snapshot["id"], version=constants.MIN_SHARE_GROUP_MICROVERSION)
        self.shares_v2_client.wait_for_resource_deletion(
            share_group_snapshot_id=sg_snapshot["id"])
        self.assertRaises(
            lib_exc.NotFound,
            self.shares_v2_client.get_share_group_snapshot,
            sg_snapshot['id'],
            version=constants.MIN_SHARE_GROUP_MICROVERSION)

    @decorators.idempotent_id('727d9c69-4c3b-4375-a91b-8b3efd349976')
    @tc.attr(base.TAG_POSITIVE, base.TAG_API_WITH_BACKEND)
    @testtools.skipUnless(CONF.share.run_snapshot_tests,
                          "Snapshot tests are disabled.")
    def test_create_share_group_from_empty_share_group_snapshot_min(self):
        # Create base share group
        share_group = self.create_share_group(
            share_group_type_id=self.share_group_type_id,
            share_type_ids=[self.share_type_id],
            cleanup_in_class=False,
            version=constants.MIN_SHARE_GROUP_MICROVERSION)

        # Create share group snapshot
        sg_snapshot = self.create_share_group_snapshot_wait_for_active(
            share_group["id"], cleanup_in_class=False,
            version=constants.MIN_SHARE_GROUP_MICROVERSION)

        snapshot_members = sg_snapshot['members']

        self.assertEmpty(
            snapshot_members,
            'Expected 0 share group snapshot members, got %s' %
            len(snapshot_members))

        new_share_group = self.create_share_group(
            share_group_type_id=self.share_group_type_id,
            cleanup_in_class=False,
            source_share_group_snapshot_id=sg_snapshot['id'],
            version=constants.MIN_SHARE_GROUP_MICROVERSION)

        new_shares = self.shares_v2_client.list_shares(
            params={'share_group_id': new_share_group['id']},
            version=constants.MIN_SHARE_GROUP_MICROVERSION)['shares']

        self.assertEmpty(
            new_shares, 'Expected 0 new shares, got %s' % len(new_shares))

        msg = ('Expected source_ishare_group_snapshot_id %s '
               'as source of share group %s' % (
                   sg_snapshot['id'],
                   new_share_group['source_share_group_snapshot_id']))
        self.assertEqual(
            new_share_group['source_share_group_snapshot_id'],
            sg_snapshot['id'],
            msg)

        msg = ('Unexpected share_types on new share group. Expected '
               '%s, got %s.' % (share_group['share_types'],
                                new_share_group['share_types']))
        self.assertEqual(
            sorted(share_group['share_types']),
            sorted(new_share_group['share_types']), msg)

        # Assert the share_network information is the same
        msg = 'Expected share_network %s as share_network of cg %s' % (
            share_group['share_network_id'],
            new_share_group['share_network_id'])
        self.assertEqual(
            share_group['share_network_id'],
            new_share_group['share_network_id'],
            msg)

        # Delete the share group snapshot and wait
        self.shares_v2_client.delete_share_group_snapshot(
            sg_snapshot["id"], version=constants.MIN_SHARE_GROUP_MICROVERSION)
        self.shares_v2_client.wait_for_resource_deletion(
            share_group_snapshot_id=sg_snapshot["id"])

        # Delete share group, so share network subnet deletion does not fail
        self.shares_v2_client.delete_share_group(
            share_group['id'], version=constants.MIN_SHARE_GROUP_MICROVERSION)

    @utils.skip_if_microversion_not_supported("2.34")
    @decorators.idempotent_id('14fd6d88-87ff-4af2-ad17-f95dbd8dcd61')
    @tc.attr(base.TAG_POSITIVE, base.TAG_API_WITH_BACKEND)
    @ddt.data(
        'sg', 'sg_and_share', 'none',
    )
    def test_create_sg_and_share_specifying_az(self, where_specify_az):
        # Get list of existing availability zones, at least one always
        # should exist
        azs = self.get_availability_zones_matching_share_type(
            self.share_type)

        sg_kwargs = {
            'share_group_type_id': self.share_group_type_id,
            'share_type_ids': [self.share_type_id],
            'version': '2.34',
            'cleanup_in_class': False,
        }
        if where_specify_az in ('sg', 'sg_and_share'):
            sg_kwargs['availability_zone'] = azs[0]

        # Create share group
        share_group = self.create_share_group(**sg_kwargs)

        # Get latest share group info
        share_group = self.shares_v2_client.get_share_group(
            share_group['id'], '2.34')['share_group']

        self.assertIn('availability_zone', share_group)
        if where_specify_az in ('sg', 'sg_and_share'):
            self.assertEqual(azs[0], share_group['availability_zone'])
        else:
            self.assertIn(share_group['availability_zone'], azs)

        # Test 'consistent_snapshot_support' as part of 2.33 API change
        self.assertIn('consistent_snapshot_support', share_group)
        self.assertIn(
            share_group['consistent_snapshot_support'], ('host', 'pool', None))

        s_kwargs = {
            'share_type_id': self.share_type_id,
            'share_group_id': share_group['id'],
            'version': '2.33',
            'cleanup_in_class': False,
        }
        if where_specify_az == 'sg_and_share':
            s_kwargs['availability_zone'] = azs[0]

        # Create share in share group
        share = self.create_share(**s_kwargs)

        # Get latest share info
        share = self.shares_v2_client.get_share(share['id'], '2.34')['share']

        # Verify that share always has the same AZ as share group does
        self.assertEqual(
            share_group['availability_zone'], share['availability_zone'])

    @utils.skip_if_microversion_not_supported("2.70")
    @tc.attr(base.TAG_POSITIVE, base.TAG_API_WITH_BACKEND)
    @testtools.skipUnless(CONF.share.multitenancy_enabled,
                          "Multitenancy is disabled.")
    @testtools.skipUnless(CONF.share.run_share_server_multiple_subnet_tests,
                          "Share server multiple subnet tests are disabled.")
    @testtools.skipIf(CONF.share.share_network_id != "",
                      "This test is not suitable for pre-existing "
                      "share networks.")
    @ddt.data(False, True)
    @decorators.idempotent_id('17fd1867-03a3-43d0-9be3-daf90b6c5e02')
    def test_create_sg_and_share_with_multiple_subnets(
        self, network_allocation_update):
        if network_allocation_update and not (
            CONF.share.run_network_allocation_update_tests):
            raise self.skipException(
                'Network allocation update tests are disabled.')
        extra_specs = {
            'driver_handles_share_servers': CONF.share.multitenancy_enabled,
            'share_server_multiple_subnet_support': True,
        }
        if network_allocation_update:
            extra_specs['network_allocation_update_support'] = True
        share_type = self.create_share_type(extra_specs=extra_specs)
        sg_type_name = data_utils.rand_name("tempest-manila")
        sg_type = self.create_share_group_type(
            name=sg_type_name, share_types=share_type['id'],
            client=self.admin_shares_v2_client)
        # Get list of existing availability zones, at least one always
        # should exist
        azs = self.get_availability_zones_matching_share_type(share_type)
        if len(azs) == 0:
            raise self.skipException(
                "No AZs were found. Make sure there is at least one "
                "configured.")
        share_network = self.shares_v2_client.get_share_network(
            self.shares_v2_client.share_network_id)['share_network']
        new_share_network_id = self.create_share_network(
            cleanup_in_class=False)['id']

        default_subnet = utils.share_network_get_default_subnet(
            share_network)
        subnet_data = {
            'neutron_net_id': default_subnet.get('neutron_net_id'),
            'neutron_subnet_id': default_subnet.get('neutron_subnet_id'),
            'share_network_id': new_share_network_id,
            'availability_zone': azs[0]
        }
        subnet1 = self.create_share_network_subnet(**subnet_data)
        if not network_allocation_update:
            subnet2 = self.create_share_network_subnet(**subnet_data)

        sg_kwargs = {
            'share_group_type_id': sg_type['id'],
            'share_type_ids': [share_type['id']],
            'share_network_id': new_share_network_id,
            'availability_zone': azs[0],
            'version': constants.MIN_SHARE_GROUP_MICROVERSION,
            'cleanup_in_class': False,
        }

        # Create share group
        share_group = self.create_share_group(**sg_kwargs)

        # Get latest share group info
        share_group = self.shares_v2_client.get_share_group(
            share_group['id'])['share_group']

        self.assertIn('availability_zone', share_group)
        self.assertEqual(azs[0], share_group['availability_zone'])

        # Test 'consistent_snapshot_support' as part of 2.33 API change
        self.assertIn('consistent_snapshot_support', share_group)
        self.assertIn(
            share_group['consistent_snapshot_support'], ('host', 'pool', None))

        share_data = {
            'share_type_id': share_type['id'],
            'share_group_id': share_group['id'],
            'share_network_id': new_share_network_id,
            'availability_zone': azs[0],
            'cleanup_in_class': False,
        }

        # Create share in share group
        share = self.create_share(**share_data)

        # Get latest share info
        share = self.admin_shares_v2_client.get_share(share['id'])['share']
        # Verify that share always has the same AZ as share group does
        self.assertEqual(
            share_group['availability_zone'], share['availability_zone'])

        # Get share server info
        share_server = self.admin_shares_v2_client.show_share_server(
            share['share_server_id'])['share_server']
        if network_allocation_update:
            waiters.wait_for_subnet_create_check(
                self.shares_v2_client, new_share_network_id,
                neutron_net_id=subnet_data['neutron_net_id'],
                neutron_subnet_id=subnet_data['neutron_subnet_id'],
                availability_zone=azs[0])

            subnet2 = self.create_share_network_subnet(**subnet_data)
            waiters.wait_for_resource_status(
                self.admin_shares_v2_client, share['share_server_id'],
                constants.SERVER_STATE_ACTIVE,
                resource_name="share_server",
                status_attr="status")
        share_server = self.admin_shares_v2_client.show_share_server(
            share['share_server_id'])['share_server']
        # Check if share server has multiple subnets
        self.assertIn(subnet1['id'], share_server['share_network_subnet_ids'])
        self.assertIn(subnet2['id'], share_server['share_network_subnet_ids'])
        # Delete share
        params = {"share_group_id": share_group['id']}
        self.shares_v2_client.delete_share(
            share['id'],
            params=params,
            version=constants.MIN_SHARE_GROUP_MICROVERSION)
        self.shares_client.wait_for_resource_deletion(share_id=share['id'])

        # Delete share group
        self.shares_v2_client.delete_share_group(
            share_group['id'], version=constants.MIN_SHARE_GROUP_MICROVERSION)
        self.shares_v2_client.wait_for_resource_deletion(
            share_group_id=share_group['id'])

        # Delete subnet
        self.shares_v2_client.delete_subnet(
            new_share_network_id, subnet1['id'])
        self.shares_v2_client.delete_subnet(
            new_share_network_id, subnet2['id'])



# Copyright 2014 OpenStack Foundation
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import ddt
from tempest import config
from tempest.lib.common.utils import data_utils
from tempest.lib import decorators
from tempest.lib import exceptions as lib_exc
from testtools import testcase as tc

from manila_tempest_tests.common import constants
from manila_tempest_tests.tests.api import base
from manila_tempest_tests import utils

CONF = config.CONF


@ddt.ddt
class ExtraSpecsAdminNegativeTest(base.BaseSharesMixedTest):

    @classmethod
    def resource_setup(cls):
        super(ExtraSpecsAdminNegativeTest, cls).resource_setup()
        cls.extra_specs = cls.add_extra_specs_to_dict({"key": "value"})

    @decorators.idempotent_id('195c1cc6-249a-4f82-b420-4901d2557b3a')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API)
    def test_try_create_extra_specs_with_user(self):
        st = self.create_share_type(extra_specs=self.extra_specs)
        self.assertRaises(
            lib_exc.Forbidden,
            self.shares_v2_client.create_share_type_extra_specs,
            st["id"],
            self.add_extra_specs_to_dict({"key": "new_value"}))

    @decorators.idempotent_id('dc883ec3-1bae-4ed7-8bf5-2cdc7027e37b')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API)
    def test_try_list_extra_specs_with_user(self):
        st = self.create_share_type(extra_specs=self.extra_specs)
        self.assertRaises(
            lib_exc.Forbidden,
            self.shares_v2_client.get_share_type_extra_specs,
            st["id"])

    @decorators.idempotent_id('1d3e687e-b2fb-4b96-8428-324ff881eea2')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API)
    def test_try_get_extra_spec_with_user(self):
        st = self.create_share_type(extra_specs=self.extra_specs)
        self.assertRaises(
            lib_exc.Forbidden,
            self.shares_v2_client.get_share_type_extra_spec,
            st["id"], "key")

    @decorators.idempotent_id('4c9505d9-d4ef-42fa-8410-8ab88ec0c852')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API)
    def test_try_get_extra_specs_with_user(self):
        st = self.create_share_type(extra_specs=self.extra_specs)
        self.assertRaises(
            lib_exc.Forbidden,
            self.shares_v2_client.get_share_type_extra_specs,
            st["id"])

    @decorators.idempotent_id('36c5ada4-9efd-4f6a-b58d-24f08a2433ce')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API)
    def test_try_read_extra_specs_on_share_type_with_user(self):
        st = self.create_share_type(extra_specs=self.extra_specs)
        share_type = self.shares_v2_client.get_share_type(st['id'])
        # Verify a non-admin can only read the required extra-specs
        expected_keys = ['driver_handles_share_servers', 'snapshot_support']
        if utils.is_microversion_ge(CONF.share.max_api_microversion, '2.24'):
            expected_keys.append('create_share_from_snapshot_support')
        if utils.is_microversion_ge(CONF.share.max_api_microversion,
                                    constants.REVERT_TO_SNAPSHOT_MICROVERSION):
            expected_keys.append('revert_to_snapshot_support')
        if utils.is_microversion_ge(CONF.share.max_api_microversion, '2.32'):
            expected_keys.append('mount_snapshot_support')
        actual_keys = share_type['share_type']['extra_specs'].keys()
        self.assertEqual(sorted(expected_keys), sorted(actual_keys),
                         'Incorrect extra specs visible to non-admin user; '
                         'expected %s, got %s' % (expected_keys, actual_keys))

    @decorators.idempotent_id('62a9b77a-f796-4bd9-baf9-7c24b3f55560')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API)
    def test_try_update_extra_spec_with_user(self):
        st = self.create_share_type(extra_specs=self.extra_specs)
        self.assertRaises(
            lib_exc.Forbidden,
            self.shares_v2_client.update_share_type_extra_spec,
            st["id"], "key", "new_value")

    @decorators.idempotent_id('207cec3c-8ed9-4d6d-8fc8-3aecaacdff93')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API)
    def test_try_update_extra_specs_with_user(self):
        st = self.create_share_type(extra_specs=self.extra_specs)
        self.assertRaises(
            lib_exc.Forbidden,
            self.shares_v2_client.update_share_type_extra_specs,
            st["id"], {"key": "new_value"})

    @decorators.idempotent_id('3f43c5d0-23c5-4b76-98c7-a3f9adb33c89')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API)
    def test_try_delete_extra_specs_with_user(self):
        st = self.create_share_type(extra_specs=self.extra_specs)
        self.assertRaises(
            lib_exc.Forbidden,
            self.shares_v2_client.delete_share_type_extra_spec,
            st["id"], "key")

    @decorators.idempotent_id('d82a7bcc-1dc5-4ef8-87cd-f8dff1574adc')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API)
    def test_try_set_too_long_key(self):
        too_big_key = "k" * 256
        st = self.create_share_type(extra_specs=self.extra_specs)
        self.assertRaises(
            lib_exc.BadRequest,
            self.admin_shares_v2_client.create_share_type_extra_specs,
            st["id"],
            self.add_extra_specs_to_dict({too_big_key: "value"}))

    @decorators.idempotent_id('210faa88-8f2f-45f5-9bcf-54ce81d03788')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API)
    def test_try_set_too_long_value_with_creation(self):
        too_big_value = "v" * 256
        st = self.create_share_type(extra_specs=self.extra_specs)
        self.assertRaises(
            lib_exc.BadRequest,
            self.admin_shares_v2_client.create_share_type_extra_specs,
            st["id"],
            self.add_extra_specs_to_dict({"key": too_big_value}))

    @decorators.idempotent_id('890dceaa-22c4-4d2c-99ca-16ac8cdda33c')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API)
    def test_try_set_too_long_value_with_update(self):
        too_big_value = "v" * 256
        st = self.create_share_type(extra_specs=self.extra_specs)
        self.admin_shares_v2_client.create_share_type_extra_specs(
            st["id"],
            self.add_extra_specs_to_dict({"key": "value"}))
        self.assertRaises(
            lib_exc.BadRequest,
            self.admin_shares_v2_client.update_share_type_extra_specs,
            st["id"],
            self.add_extra_specs_to_dict({"key": too_big_value}))

    @decorators.idempotent_id('c512437e-6cfd-4545-859f-554955bd0fc9')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API)
    def test_try_set_too_long_value_with_update_of_one_key(self):
        too_big_value = "v" * 256
        st = self.create_share_type(extra_specs=self.extra_specs)
        self.admin_shares_v2_client.create_share_type_extra_specs(
            st["id"],
            self.add_extra_specs_to_dict({"key": "value"}))
        self.assertRaises(
            lib_exc.BadRequest,
            self.admin_shares_v2_client.update_share_type_extra_spec,
            st["id"], "key", too_big_value)

    @decorators.idempotent_id('938115f7-512e-49c3-a16a-a6498d6069bb')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API)
    def test_try_list_es_with_empty_shr_type_id(self):
        self.assertRaises(
            lib_exc.NotFound,
            self.admin_shares_v2_client.get_share_type_extra_specs, "")

    @decorators.idempotent_id('65c3c528-008f-43f5-b367-8bd77cfab4a7')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API)
    def test_try_list_es_with_invalid_shr_type_id(self):
        self.assertRaises(
            lib_exc.NotFound,
            self.admin_shares_v2_client.get_share_type_extra_specs,
            data_utils.rand_name("fake"))

    @decorators.idempotent_id('430e0bed-9073-4822-8d0b-a3c55a8dfa31')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API)
    def test_try_create_es_with_empty_shr_type_id(self):
        self.assertRaises(
            lib_exc.NotFound,
            self.admin_shares_v2_client.create_share_type_extra_specs,
            "", {"key1": "value1", })

    @decorators.idempotent_id('7b43025d-b014-460f-9c9d-1004433fb798')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API)
    def test_try_create_es_with_invalid_shr_type_id(self):
        self.assertRaises(
            lib_exc.NotFound,
            self.admin_shares_v2_client.create_share_type_extra_specs,
            data_utils.rand_name("fake"), {"key1": "value1", })

    @decorators.idempotent_id('7b9bee14-5ca5-4110-a56a-b3030b6b3948')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API)
    def test_try_create_es_with_empty_specs(self):
        st = self.create_share_type(extra_specs=self.extra_specs)
        self.assertRaises(
            lib_exc.BadRequest,
            self.admin_shares_v2_client.create_share_type_extra_specs,
            st["id"], "")

    @decorators.idempotent_id('7f199925-44d2-4d92-bedc-2636c07621fb')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API)
    def test_try_create_es_with_invalid_specs(self):
        st = self.create_share_type(extra_specs=self.extra_specs)
        self.assertRaises(
            lib_exc.BadRequest,
            self.admin_shares_v2_client.create_share_type_extra_specs,
            st["id"], {"": "value_with_empty_key"})

    @decorators.idempotent_id('51241ed9-350b-4218-bfb0-c446d660d70b')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API)
    def test_try_get_extra_spec_with_empty_key(self):
        st = self.create_share_type(extra_specs=self.extra_specs)
        self.assertRaises(
            lib_exc.NotFound,
            self.admin_shares_v2_client.get_share_type_extra_spec,
            st["id"], "")

    @decorators.idempotent_id('271d825b-2c57-429a-8dca-2cb9dd140dd0')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API)
    def test_try_get_extra_spec_with_invalid_key(self):
        st = self.create_share_type(extra_specs=self.extra_specs)
        self.assertRaises(
            lib_exc.NotFound,
            self.admin_shares_v2_client.get_share_type_extra_spec,
            st["id"], data_utils.rand_name("fake"))

    @decorators.idempotent_id('cf07e7bc-cd3a-4c85-a848-786a60ba1f7d')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API)
    def test_try_get_extra_specs_with_empty_shr_type_id(self):
        self.assertRaises(
            lib_exc.NotFound,
            self.admin_shares_v2_client.get_share_type_extra_specs,
            "")

    @decorators.idempotent_id('dfe6e34b-c86a-4cf6-9bf3-19de3a886b67')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API)
    def test_try_get_extra_specs_with_invalid_shr_type_id(self):
        self.assertRaises(
            lib_exc.NotFound,
            self.admin_shares_v2_client.get_share_type_extra_specs,
            data_utils.rand_name("fake"))

    @decorators.idempotent_id('0b8f6b51-0583-4b59-a851-9189db657a05')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API)
    def test_try_delete_es_key_with_empty_shr_type_id(self):
        self.assertRaises(
            lib_exc.NotFound,
            self.admin_shares_v2_client.delete_share_type_extra_spec,
            "", "key", )

    @decorators.idempotent_id('9aa8fb2a-28a5-44b7-848b-9b49e1e9d670')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API)
    def test_try_delete_es_key_with_invalid_shr_type_id(self):
        self.assertRaises(
            lib_exc.NotFound,
            self.admin_shares_v2_client.delete_share_type_extra_spec,
            data_utils.rand_name("fake"), "key", )

    @decorators.idempotent_id('cd68d020-24d2-4f68-8691-782b4815c1b0')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API)
    def test_try_delete_with_invalid_key(self):
        st = self.create_share_type(extra_specs=self.extra_specs)
        self.assertRaises(
            lib_exc.NotFound,
            self.admin_shares_v2_client.delete_share_type_extra_spec,
            st["id"], data_utils.rand_name("fake"))

    @decorators.idempotent_id('1ed5cbc9-21a8-45b7-8069-b7c0aaeede21')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API)
    def test_try_update_spec_with_empty_shr_type_id(self):
        self.assertRaises(
            lib_exc.NotFound,
            self.admin_shares_v2_client.update_share_type_extra_spec,
            "", "key", "new_value")

    @decorators.idempotent_id('704087fa-0397-4d9d-98b7-f9b081f64c86')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API)
    def test_try_update_spec_with_invalid_shr_type_id(self):
        self.assertRaises(
            lib_exc.NotFound,
            self.admin_shares_v2_client.update_share_type_extra_spec,
            data_utils.rand_name("fake"), "key", "new_value")

    @decorators.idempotent_id('eab96e92-9b95-44b0-89a2-e907a103039d')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API)
    def test_try_update_spec_with_empty_key(self):
        st = self.create_share_type(extra_specs=self.extra_specs)
        self.assertRaises(
            lib_exc.NotFound,
            self.admin_shares_v2_client.update_share_type_extra_spec,
            st["id"], "", "new_value")

    @decorators.idempotent_id('70af1a8a-ab3e-4c8b-862d-8a36c2d47cb3')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API)
    def test_try_update_with_invalid_shr_type_id(self):
        self.assertRaises(
            lib_exc.NotFound,
            self.admin_shares_v2_client.update_share_type_extra_specs,
            data_utils.rand_name("fake"), {"key": "new_value"})

    @decorators.idempotent_id('d2595594-eaad-43dc-b847-0a009a17d854')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API)
    def test_try_update_with_invalid_specs(self):
        st = self.create_share_type(extra_specs=self.extra_specs)
        self.assertRaises(
            lib_exc.BadRequest,
            self.admin_shares_v2_client.update_share_type_extra_specs,
            st["id"], {"": "new_value"})

    @decorators.idempotent_id('6849eada-89a8-4009-a91d-87367621f9aa')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API)
    def test_try_delete_spec_driver_handles_share_servers(self):
        st = self.create_share_type(extra_specs=self.extra_specs)

        # Try delete extra spec 'driver_handles_share_servers'
        self.assertRaises(
            lib_exc.Forbidden,
            self.admin_shares_v2_client.delete_share_type_extra_spec,
            st["id"],
            "driver_handles_share_servers")

    @decorators.idempotent_id('6ea50e81-2c93-4258-8358-6f8d354a339a')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API)
    @ddt.data('2.0', '2.23')
    def test_try_delete_required_spec_snapshot_support_version(self, version):
        utils.check_skip_if_microversion_not_supported(version)
        st = self.create_share_type(extra_specs=self.extra_specs)
        # Try delete extra spec 'snapshot_support'
        self.assertRaises(
            lib_exc.Forbidden,
            self.admin_shares_v2_client.delete_share_type_extra_spec,
            st["id"], "snapshot_support", version=version)



# Copyright 2015 Mirantis Inc.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from tempest.lib import decorators
from tempest.lib import exceptions as lib_exc
from testtools import testcase as tc

from manila_tempest_tests.tests.api import base
from manila_tempest_tests import utils


class AvailabilityZonesNegativeTest(base.BaseSharesTest):

    @classmethod
    def skip_checks(cls):
        super(AvailabilityZonesNegativeTest, cls).skip_checks()
        utils.check_skip_if_microversion_not_supported('2.7')

    @decorators.idempotent_id('050fa9fb-4cc8-4064-9e91-bb169fca03db')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API)
    def test_list_availability_zones_api_not_found_with_legacy_url(self):
        # NOTE(vponomaryov): remove this test with removal of availability zone
        # extension url support.
        self.assertRaises(
            lib_exc.NotFound,
            self.shares_v2_client.list_availability_zones,
            url='os-availability-zone',
            version='2.7',
        )

    @decorators.idempotent_id('133ffc40-bff1-491a-a230-189cddd66e29')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API)
    def test_list_availability_zones_api_not_found(self):
        self.assertRaises(
            lib_exc.NotFound,
            self.shares_v2_client.list_availability_zones,
            url='availability-zones',
            version='2.6',
        )



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from tempest import config
from tempest.lib import decorators
from tempest.lib import exceptions as lib_exc
import testtools
from testtools import testcase as tc

from manila_tempest_tests.common import constants
from manila_tempest_tests.common import waiters
from manila_tempest_tests import share_exceptions
from manila_tempest_tests.tests.api import base
from manila_tempest_tests import utils

CONF = config.CONF
_MIN_SUPPORTED_MICROVERSION = '2.11'


class ReplicationActionsAdminTest(base.BaseSharesMixedTest):

    @classmethod
    def skip_checks(cls):
        super(ReplicationActionsAdminTest, cls).skip_checks()
        if not CONF.share.run_replication_tests:
            raise cls.skipException('Replication tests are disabled.')
        if CONF.share.multitenancy_enabled:
            raise cls.skipException(
                'Only for driver_handles_share_servers = False driver mode.')

        utils.check_skip_if_microversion_not_supported(
            _MIN_SUPPORTED_MICROVERSION)

    @classmethod
    def resource_setup(cls):
        super(ReplicationActionsAdminTest, cls).resource_setup()
        cls.admin_client = cls.admin_shares_v2_client
        cls.member_client = cls.shares_v2_client
        cls.replication_type = CONF.share.backend_replication_type
        cls.multitenancy_enabled = (
            utils.replication_with_multitenancy_support())

        if cls.replication_type not in constants.REPLICATION_TYPE_CHOICES:
            raise share_exceptions.ShareReplicationTypeException(
                replication_type=cls.replication_type
            )

        # create share type
        extra_specs = {"replication_type": cls.replication_type}
        if CONF.share.capability_snapshot_support:
            extra_specs.update({"snapshot_support": True})
        cls.share_type = cls.create_share_type(extra_specs=extra_specs)
        cls.share_type_id = cls.share_type['id']

        cls.sn_id = None
        if cls.multitenancy_enabled:
            cls.share_network = cls.shares_v2_client.get_share_network(
                cls.shares_v2_client.share_network_id)['share_network']
            cls.sn_id = cls.share_network['id']

        cls.zones = cls.get_availability_zones_matching_share_type(
            cls.share_type, client=cls.admin_client)
        cls.share_zone = cls.zones[0]
        cls.replica_zone = cls.zones[-1]

        # create share
        cls.share = cls.create_share(size=CONF.share.share_size + 1,
                                     share_type_id=cls.share_type_id,
                                     availability_zone=cls.share_zone,
                                     share_network_id=cls.sn_id,
                                     client=cls.admin_client)
        cls.replica = cls.admin_client.list_share_replicas(
            share_id=cls.share['id'])['share_replicas'][0]

    @decorators.idempotent_id('b39f319e-2515-42c0-85c4-21c2fb2123bf')
    @tc.attr(base.TAG_POSITIVE, base.TAG_API_WITH_BACKEND)
    @testtools.skipUnless(CONF.share.run_extend_tests,
                          'Extend share tests are disabled.')
    def test_extend_replicated_share(self):
        # Test extend share
        new_size = self.share["size"] + 1
        self.admin_client.extend_share(self.share["id"], new_size)
        waiters.wait_for_resource_status(
            self.admin_client, self.share["id"], "available")
        share = self.admin_client.get_share(self.share["id"])['share']
        self.assertEqual(new_size, int(share["size"]))

    @decorators.idempotent_id('743bfb8e-a314-4e8e-92b5-079bd3eae72d')
    @tc.attr(base.TAG_POSITIVE, base.TAG_API_WITH_BACKEND)
    @testtools.skipUnless(CONF.share.run_shrink_tests,
                          'Shrink share tests are disabled.')
    def test_shrink_replicated_share(self):
        share = self.admin_client.get_share(self.share["id"])['share']
        new_size = self.share["size"] - 1
        self.admin_client.shrink_share(self.share["id"], new_size)
        waiters.wait_for_resource_status(
            self.admin_client, share["id"], "available")
        shrink_share = self.admin_client.get_share(self.share["id"])['share']
        self.assertEqual(new_size, int(shrink_share["size"]))

    @decorators.idempotent_id('84150cd6-2777-4806-8aa3-51359f16816e')
    @tc.attr(base.TAG_POSITIVE, base.TAG_BACKEND)
    @testtools.skipUnless(CONF.share.run_manage_unmanage_tests,
                          'Manage/Unmanage Tests are disabled.')
    def test_manage_share_for_replication_type(self):
        """Manage a share with replication share type."""
        # Create a share and unmanage it
        share = self.create_share(size=CONF.share.share_size,
                                  share_type_id=self.share_type_id,
                                  availability_zone=self.share_zone,
                                  share_network_id=self.sn_id,
                                  cleanup_in_class=True,
                                  client=self.admin_client)
        share = self.admin_client.get_share(share["id"])['share']
        export_locations = self.admin_client.list_share_export_locations(
            share["id"])['export_locations']
        export_path = export_locations[0]['path']

        self.admin_client.unmanage_share(share['id'])
        self.admin_client.wait_for_resource_deletion(share_id=share['id'])

        # Manage the previously unmanaged share
        managed_share = self.admin_client.manage_share(
            share['host'], share['share_proto'],
            export_path, self.share_type_id)['share']
        waiters.wait_for_resource_status(
            self.admin_client, managed_share['id'], 'available')

        # Add managed share to cleanup queue
        self.method_resources.insert(
            0, {'type': 'share', 'id': managed_share['id'],
                'client': self.admin_client})

        # Make sure a replica can be added to newly managed share
        self.create_share_replica(managed_share['id'], self.replica_zone,
                                  cleanup=True, client=self.admin_client)

    @decorators.idempotent_id('cbbe2650-47bb-456b-8b41-74c66270ea97')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API_WITH_BACKEND)
    @testtools.skipUnless(CONF.share.run_manage_unmanage_tests,
                          'Manage/Unmanage Tests are disabled.')
    def test_unmanage_replicated_share_with_replica(self):
        """Try to unmanage a share having replica."""
        # Create a share replica before unmanaging the share
        self.create_share_replica(self.share["id"], self.replica_zone,
                                  cleanup=True, client=self.admin_client)
        self.assertRaises(
            lib_exc.Conflict,
            self.admin_client.unmanage_share,
            share_id=self.share['id'])

    @decorators.idempotent_id('796fb2a8-1ac3-4eee-b12e-da511eb52e87')
    @tc.attr(base.TAG_POSITIVE, base.TAG_BACKEND)
    @testtools.skipUnless(CONF.share.run_manage_unmanage_tests,
                          'Manage/Unmanage Tests are disabled.')
    def test_unmanage_replicated_share_with_no_replica(self):
        """Unmanage a replication type share that does not have replica."""
        share = self.create_share(size=CONF.share.share_size,
                                  share_type_id=self.share_type_id,
                                  share_network_id=self.sn_id,
                                  availability_zone=self.share_zone,
                                  client=self.admin_client)
        self.admin_client.unmanage_share(share['id'])
        self.admin_client.wait_for_resource_deletion(share_id=share['id'])

    @decorators.idempotent_id('93220873-c6c4-40f7-840d-d0ff02e7cd7e')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API_WITH_BACKEND)
    @testtools.skipUnless(CONF.share.run_manage_unmanage_snapshot_tests,
                          'Manage/Unmanage Snapshot Tests are disabled.')
    def test_manage_replicated_share_snapshot(self):
        """Try to manage a snapshot of the replicated."""
        # Create a share replica before managing the snapshot
        self.create_share_replica(self.share["id"], self.replica_zone,
                                  cleanup=True, client=self.admin_client)
        self.assertRaises(
            lib_exc.Conflict,
            self.admin_client.manage_snapshot,
            share_id=self.share['id'],
            provider_location="127.0.0.1:/fake_provider_location/"
                              "manila_share_9dc61f49_fbc8_48d7_9337_2f9593d9")

    @decorators.idempotent_id('0b0cd350-8691-477b-adb1-5e79b92e3759')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API_WITH_BACKEND)
    @testtools.skipUnless(CONF.share.run_manage_unmanage_snapshot_tests,
                          'Manage/Unmanage Snapshot Tests are disabled.')
    def test_unmanage_replicated_share_snapshot(self):
        """Try to unmanage a snapshot of the replicated share with replica."""
        # Create a share replica before unmanaging the snapshot
        self.create_share_replica(self.share["id"], self.replica_zone,
                                  cleanup=True, client=self.admin_client)
        snapshot = self.create_snapshot_wait_for_active(
            self.share["id"], client=self.admin_client)
        self.assertRaises(
            lib_exc.Conflict,
            self.admin_client.unmanage_snapshot,
            snapshot_id=snapshot['id'])



# Copyright 2016 Andrew Kerr
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import itertools

import ddt
from tempest import config
from tempest.lib.common.utils import data_utils
from tempest.lib import decorators
from testtools import testcase as tc

from manila_tempest_tests.common import constants
from manila_tempest_tests.tests.api import base
from manila_tempest_tests import utils


CONF = config.CONF

LATEST_MICROVERSION = CONF.share.max_api_microversion


@ddt.ddt
class ShareGroupTypesTest(base.BaseSharesAdminTest):

    @classmethod
    def skip_checks(cls):
        super(ShareGroupTypesTest, cls).skip_checks()
        if not CONF.share.run_share_group_tests:
            raise cls.skipException('Share Group tests disabled.')

        utils.check_skip_if_microversion_not_supported(
            constants.MIN_SHARE_GROUP_MICROVERSION)

    @classmethod
    def resource_setup(cls):
        super(ShareGroupTypesTest, cls).resource_setup()

        # Create 2 share_types
        name = data_utils.rand_name("tempest-manila")
        extra_specs = cls.add_extra_specs_to_dict()
        cls.share_type = cls.create_share_type(name, extra_specs=extra_specs)

        name = data_utils.rand_name("tempest-manila")
        cls.share_type2 = cls.create_share_type(name, extra_specs=extra_specs)

    @decorators.idempotent_id('e2ba1754-cecc-4178-ad39-eefbb59e4d6d')
    @tc.attr(base.TAG_POSITIVE, base.TAG_API)
    @ddt.data(
        *itertools.product(('id', 'name'), utils.deduplicate(
            [LATEST_MICROVERSION, constants.MIN_SHARE_GROUP_MICROVERSION,
             constants.SHARE_GROUPS_GRADUATION_VERSION])))
    @ddt.unpack
    def test_create_get_delete_share_group_type(self, st_key, version):
        utils.check_skip_if_microversion_not_supported(version)
        name = data_utils.rand_name("tempest-manila")

        # Create share group type
        sg_type_c = self.create_share_group_type(
            name=name,
            share_types=self.share_type[st_key],
            cleanup_in_class=False,
            version=version)

        self.assertEqual(
            [self.share_type['id']],
            sg_type_c['share_types'],
            'Share type not applied correctly.')

        # Read share group type
        sg_type_r = self.shares_v2_client.get_share_group_type(
            sg_type_c['id'], version=version)['share_group_type']
        keys = set(sg_type_r.keys())
        self.assertTrue(
            constants.SHARE_GROUP_TYPE_REQUIRED_KEYS.issubset(keys),
            'At least one expected key missing from share group type '
            'response. Expected %s, got %s.' % (
                constants.SHARE_GROUP_TYPE_REQUIRED_KEYS, keys))
        self.assertEqual(sg_type_c['name'], sg_type_r['name'])

        # Delete share group type
        self.shares_v2_client.delete_share_group_type(
            sg_type_r['id'], version=version)
        self.shares_v2_client.wait_for_resource_deletion(
            share_group_type_id=sg_type_r['id'])

    @decorators.idempotent_id('65cf04d6-0620-4e0d-96f2-456946e8a552')
    @tc.attr(base.TAG_POSITIVE, base.TAG_API)
    @ddt.data('id', 'name')
    def test_create_share_group_type_multiple_share_types_min(self, st_key):
        name = data_utils.rand_name("tempest-manila")

        sg_type = self.create_share_group_type(
            name=name,
            share_types=[self.share_type[st_key], self.share_type2[st_key]],
            cleanup_in_class=False,
            version=constants.MIN_SHARE_GROUP_MICROVERSION)

        self.assertEqual(
            {self.share_type['id'], self.share_type2['id']},
            set(sg_type['share_types']),
            'Share types not applied correctly.')

    @decorators.idempotent_id('6e103b7b-cd01-4edb-91e7-a16681781b36')
    @tc.attr(base.TAG_POSITIVE, base.TAG_API)
    def test_create_share_group_type_with_one_spec_min(self):
        name = data_utils.rand_name("tempest-manila")
        group_specs = {'key': 'value'}

        sg_type = self.create_share_group_type(
            name=name,
            share_types=self.share_type['id'],
            group_specs=group_specs,
            cleanup_in_class=False,
            version=constants.MIN_SHARE_GROUP_MICROVERSION)

        self.assertDictMatch(group_specs, sg_type['group_specs'])

    @decorators.idempotent_id('ec459ebb-5c45-4d4f-95f0-3bbf2974366c')
    @tc.attr(base.TAG_POSITIVE, base.TAG_API)
    def test_create_share_group_type_with_multiple_specs_min(self):
        name = data_utils.rand_name("tempest-manila")
        group_specs = {'key1': 'value1', 'key2': 'value2'}

        sg_type = self.create_share_group_type(
            name=name,
            share_types=self.share_type['id'],
            group_specs=group_specs,
            cleanup_in_class=False,
            version=constants.MIN_SHARE_GROUP_MICROVERSION)

        self.assertDictMatch(group_specs, sg_type['group_specs'])

    @decorators.idempotent_id('dd620bfd-197b-4675-ace6-e26f809bb26e')
    @tc.attr(base.TAG_POSITIVE, base.TAG_API)
    def test_get_one_share_group_type_extra_spec(self):
        name = data_utils.rand_name('share-group-type')
        group_specs = {'key1': 'value1', 'key2': 'value2'}

        sg_type = self.create_share_group_type(
            name=name,
            share_types=self.share_type['id'],
            group_specs=group_specs,
            cleanup_in_class=False,
            version=constants.MIN_SHARE_GROUP_MICROVERSION)

        extra_spec = self.shares_v2_client.get_share_group_type_spec(
            sg_type['id'], 'key1')

        self.assertEqual({'key1': group_specs['key1']}, extra_spec)

    @decorators.idempotent_id('eef69171-9757-423c-8cd4-487cbd84ca24')
    @tc.attr(base.TAG_POSITIVE, base.TAG_API)
    def test_get_all_share_group_type_extra_specs(self):
        name = data_utils.rand_name('share-group-type')
        group_specs = {'key': 'value'}

        sg_type = self.create_share_group_type(
            name=name,
            share_types=self.share_type['id'],
            group_specs=group_specs,
            cleanup_in_class=False,
            version=constants.MIN_SHARE_GROUP_MICROVERSION)

        extra_specs = self.shares_v2_client.get_share_group_type_specs(
            sg_type['id'])

        self.assertDictMatch(group_specs, extra_specs['group_specs'])

    @decorators.idempotent_id('15b44580-a34d-4e0d-a77b-0e76b45d6199')
    @tc.attr(base.TAG_POSITIVE, base.TAG_API)
    @ddt.data(
        *utils.deduplicate([constants.MIN_SHARE_GROUP_MICROVERSION,
                            constants.SHARE_GROUPS_GRADUATION_VERSION,
                            LATEST_MICROVERSION]))
    def test_update_single_share_group_type_spec(self, version):
        utils.check_skip_if_microversion_not_supported(version)
        name = data_utils.rand_name("tempest-manila")
        group_specs = {'key1': 'value1', 'key2': 'value2'}

        sg_type = self.create_share_group_type(
            name=name,
            share_types=self.share_type['id'],
            group_specs=group_specs,
            cleanup_in_class=False,
            version=version)

        self.assertDictMatch(group_specs, sg_type['group_specs'])

        group_specs = {'key1': 'value1', 'key2': 'value2'}

        self.shares_v2_client.update_share_group_type_spec(
            sg_type['id'], 'key1', 'value3', version=version)
        sg_type = self.shares_v2_client.get_share_group_type(
            sg_type['id'])['share_group_type']

        self.assertIn('key1', sg_type['group_specs'])
        self.assertIn('key2', sg_type['group_specs'])
        self.assertEqual('value3', sg_type['group_specs']['key1'])
        self.assertEqual(group_specs['key2'], sg_type['group_specs']['key2'])

    @decorators.idempotent_id('3ed054c5-d0ab-4a73-b704-994eb7f9c46c')
    @tc.attr(base.TAG_POSITIVE, base.TAG_API)
    def test_update_all_share_group_type_specs_min(self):
        name = data_utils.rand_name("tempest-manila")
        group_specs = {'key1': 'value1', 'key2': 'value2'}

        sg_type = self.create_share_group_type(
            name=name,
            share_types=self.share_type['id'],
            group_specs=group_specs,
            cleanup_in_class=False,
            version=constants.MIN_SHARE_GROUP_MICROVERSION)

        self.assertDictMatch(group_specs, sg_type['group_specs'])

        group_specs = {'key1': 'value3', 'key2': 'value4'}

        self.shares_v2_client.update_share_group_type_specs(
            sg_type['id'], group_specs)
        sg_type = self.shares_v2_client.get_share_group_type(
            sg_type['id'])['share_group_type']

        for k, v in group_specs.items():
            self.assertIn(k, sg_type['group_specs'])
            self.assertEqual(v, sg_type['group_specs'][k])

    @decorators.idempotent_id('efddee69-ca23-4681-8247-94ded81c4c3a')
    @tc.attr(base.TAG_POSITIVE, base.TAG_API)
    @ddt.data(
        *utils.deduplicate([constants.MIN_SHARE_GROUP_MICROVERSION,
                            constants.SHARE_GROUPS_GRADUATION_VERSION,
                            LATEST_MICROVERSION]))
    def test_delete_single_share_group_type_spec_min(self, version):
        utils.check_skip_if_microversion_not_supported(version)
        name = data_utils.rand_name("tempest-manila")
        group_specs = {'key1': 'value1', 'key2': 'value2'}

        sg_type = self.create_share_group_type(
            name=name,
            share_types=self.share_type['id'],
            group_specs=group_specs,
            cleanup_in_class=False,
            version=version)

        self.assertDictMatch(group_specs, sg_type['group_specs'])

        key_to_delete = 'key1'
        group_specs.pop(key_to_delete)

        self.shares_v2_client.delete_share_group_type_spec(
            sg_type['id'], key_to_delete, version=version)
        sg_type = self.shares_v2_client.get_share_group_type(
            sg_type['id'], version=version)['share_group_type']

        self.assertDictMatch(group_specs, sg_type['group_specs'])

    @decorators.idempotent_id('c2d34b42-e3ec-404e-8b7a-0fe9b1560507')
    @tc.attr(base.TAG_POSITIVE, base.TAG_API)
    @ddt.data(
        *utils.deduplicate([constants.MIN_SHARE_GROUP_MICROVERSION,
                            constants.SHARE_GROUPS_GRADUATION_VERSION,
                            LATEST_MICROVERSION]))
    def test_private_share_group_type_access(self, version):
        utils.check_skip_if_microversion_not_supported(version)
        name = data_utils.rand_name("tempest-manila")
        group_specs = {"key1": "value1", "key2": "value2"}
        project_id = self.shares_v2_client.tenant_id

        # Create private share group type
        sgt_create = self.create_share_group_type(
            name=name,
            share_types=[self.share_type['id']],
            is_public=False,
            group_specs=group_specs,
            version=version
        )
        self.assertEqual(name, sgt_create['name'])
        sgt_id = sgt_create["id"]

        # It should not be listed without access
        sgt_list = self.shares_v2_client.list_share_group_types(
            version=version)['share_group_types']
        self.assertFalse(any(sgt_id == sgt["id"] for sgt in sgt_list))

        # List projects that have access for share group type - none expected
        access = self.shares_v2_client.list_access_to_share_group_type(
            sgt_id, version=version)['share_group_type_access']
        self.assertEmpty(access)

        # Add project access to share group type
        access = self.shares_v2_client.add_access_to_share_group_type(
            sgt_id, project_id, version=version)

        # Now it should be listed
        sgt_list = self.shares_v2_client.list_share_group_types(
            version=version)['share_group_types']
        self.assertTrue(any(sgt_id == sgt["id"] for sgt in sgt_list))

        # List projects that have access for share group type - one expected
        access = self.shares_v2_client.list_access_to_share_group_type(
            sgt_id, version=version)['share_group_type_access']
        expected = [{'share_group_type_id': sgt_id, 'project_id': project_id}]
        self.assertEqual(expected, access)

        # Remove project access from share group type
        access = self.shares_v2_client.remove_access_from_share_group_type(
            sgt_id, project_id, version=version)

        # It should not be listed without access
        sgt_list = self.shares_v2_client.list_share_group_types(
            version=version)['share_group_types']
        self.assertFalse(any(sgt_id == sgt["id"] for sgt in sgt_list))

        # List projects that have access for share group type - none expected
        access = self.shares_v2_client.list_access_to_share_group_type(
            sgt_id, version=version)['share_group_type_access']
        self.assertEmpty(access)

    @decorators.idempotent_id('b8b20a96-cecc-4677-8a77-aae3b93e5b96')
    @tc.attr(base.TAG_POSITIVE, base.TAG_API)
    @ddt.data(*utils.deduplicate(('2.45', '2.46', LATEST_MICROVERSION)))
    def test_share_group_type_create_show_list_with_is_default_key(self,
                                                                   version):
        utils.check_skip_if_microversion_not_supported(version)
        name = data_utils.rand_name("tempest-manila")

        # Create share group type
        sg_type_c = self.create_share_group_type(
            name=name,
            share_types=self.share_type['id'],
            cleanup_in_class=False,
            version=version)
        if utils.is_microversion_ge(version, '2.46'):
            self.assertIn('is_default', sg_type_c)
            self.assertIs(False, sg_type_c['is_default'])
        else:
            self.assertNotIn('is_default', sg_type_c)

        # List share group type
        sg_type_list = self.shares_v2_client.list_share_group_types(
            version=version)['share_group_types']
        for sg_type_get in sg_type_list:
            if utils.is_microversion_ge(version, '2.46'):
                self.assertIn('is_default', sg_type_get)
                self.assertTrue(sg_type_get['is_default'] in (True, False))
            else:
                self.assertNotIn('is_default', sg_type_get)

        # Show share group type
        sg_type_id = sg_type_c['id']
        sg_type_show = self.shares_v2_client.get_share_group_type(
            sg_type_id, version=version)['share_group_type']
        if utils.is_microversion_ge(version, '2.46'):
            self.assertIn('is_default', sg_type_show)
            self.assertIs(False, sg_type_show['is_default'])
        else:
            self.assertNotIn('is_default', sg_type_show)



# Copyright (c) 2015 Clinton Knight.  All rights reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import ddt
from tempest import config
from tempest.lib.common.utils import data_utils
from tempest.lib import decorators
from testtools import testcase as tc

from manila_tempest_tests.tests.api import base
from manila_tempest_tests import utils

CONF = config.CONF


@ddt.ddt
class SchedulerStatsAdminTest(base.BaseSharesAdminTest):

    @classmethod
    def _create_share_type(cls, negative=False):
        name = data_utils.rand_name("unique_st_name")
        extra_specs = None

        if negative:
            extra_specs = {
                'share_backend_name': data_utils.rand_name("fake_name"),
            }

        extra_specs = cls.add_extra_specs_to_dict(extra_specs=extra_specs)
        return cls.create_share_type(
            name, extra_specs=extra_specs,
            client=cls.admin_client)

    @classmethod
    def resource_setup(cls):
        super(SchedulerStatsAdminTest, cls).resource_setup()
        cls.admin_client = cls.shares_v2_client

    @decorators.idempotent_id('dab6d654-7d1c-4bc0-8017-1c4c072b49a1')
    @tc.attr(base.TAG_POSITIVE, base.TAG_API)
    def test_pool_list(self):

        # List pools
        pool_response = self.shares_client.list_pools()
        pool_list = pool_response.get('pools')
        self.assertIsNotNone(pool_list, 'No pools returned from pools API')
        self.assertNotEmpty(pool_list)
        pool = pool_list[0]
        required_keys = {'name', 'host', 'backend', 'pool'}
        actual_keys = set(pool.keys())
        self.assertTrue(actual_keys.issuperset(required_keys))

    @decorators.idempotent_id('8f4582c4-df82-4b64-be79-a8c6fdc58eac')
    @tc.attr(base.TAG_POSITIVE, base.TAG_API)
    def test_pool_list_with_filters(self):

        # List pools
        pool_response = self.shares_client.list_pools()
        pool_list = pool_response.get('pools')

        # Ensure we got at least one pool
        self.assertIsNotNone(pool_list, 'No pools returned from pools API')
        self.assertNotEmpty(pool_list)
        pool = pool_list[0]

        # Build search opts from data and get pools again with filter
        search_opts = {
            'host': self._wrap_regex_for_exact_match(pool.get('host')),
            'backend': self._wrap_regex_for_exact_match(pool.get('backend')),
            'pool': self._wrap_regex_for_exact_match(pool.get('pool')),
        }
        pool_response = self.shares_client.list_pools(
            search_opts=search_opts)
        filtered_pool_list = pool_response.get('pools')

        # Ensure we got exactly one pool matching the first one from above
        self.assertEqual(1, len(filtered_pool_list))

        # Match the key values, not the timestamp.
        for k, v in search_opts.items():
            self.assertEqual(v[1:-1], filtered_pool_list[0][k])

    @decorators.idempotent_id('1c2f1b0f-c509-4676-8335-bb6b10debea2')
    @tc.attr(base.TAG_POSITIVE, base.TAG_API)
    def test_pool_list_with_filters_negative(self):

        # Build search opts for a non-existent pool
        search_opts = {
            'host': 'foo',
            'backend': 'bar',
            'pool': 'shark',
        }
        pool_response = self.shares_client.list_pools(
            search_opts=search_opts)
        pool_list = pool_response.get('pools')

        # Ensure we got no pools
        self.assertEmpty(pool_list)

    @decorators.idempotent_id('b6fdd156-3923-495a-ab01-f94bfb5d7040')
    @tc.attr(base.TAG_POSITIVE, base.TAG_API)
    def test_pool_list_detail(self):

        # List pools
        pool_response = self.shares_client.list_pools(detail=True)
        pool_list = pool_response.get('pools')
        self.assertIsNotNone(pool_list, 'No pools returned from pools API')
        self.assertNotEmpty(pool_list)
        pool = pool_list[0]
        required_keys = {'name', 'host', 'backend', 'pool', 'capabilities'}
        actual_keys = set(pool.keys())
        self.assertTrue(actual_keys.issuperset(required_keys))

    @decorators.idempotent_id('071a1e47-959e-47df-ad19-335064bb25ff')
    @tc.attr(base.TAG_POSITIVE, base.TAG_API)
    def test_pool_list_detail_with_filters(self):

        # List pools
        pool_response = self.shares_client.list_pools(detail=True)
        pool_list = pool_response.get('pools')

        # Ensure we got at least one pool
        self.assertIsNotNone(pool_list, 'No pools returned from pools API')
        self.assertNotEmpty(pool_list)
        pool = pool_list[0]

        # Build search opts from data and get pools again with filter
        search_opts = {
            'host': self._wrap_regex_for_exact_match(pool.get('host')),
            'backend': self._wrap_regex_for_exact_match(pool.get('backend')),
            'pool': self._wrap_regex_for_exact_match(pool.get('pool')),
        }
        pool_response = self.shares_client.list_pools(
            detail=True, search_opts=search_opts)
        filtered_pool_list = pool_response.get('pools')

        # Ensure we got exactly one pool matching the first one from above
        self.assertEqual(1, len(filtered_pool_list))

        # Match the key values, not the timestamp.
        for k, v in search_opts.items():
            self.assertEqual(v[1:-1], filtered_pool_list[0][k])

    @decorators.idempotent_id('56f04711-7b57-4d2a-a4f3-96c82845208e')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API)
    def test_pool_list_detail_with_filters_negative(self):

        # Build search opts for a non-existent pool
        search_opts = {
            'host': 'foo',
            'backend': 'bar',
            'pool': 'shark',
        }
        pool_response = self.shares_client.list_pools(
            detail=True, search_opts=search_opts)
        pool_list = pool_response.get('pools')

        # Ensure we got no pools
        self.assertEmpty(pool_list)

    @decorators.idempotent_id('f027fb62-1ec3-4f52-a782-e9dd9db34fda')
    @tc.attr(base.TAG_POSITIVE, base.TAG_API_WITH_BACKEND)
    @utils.skip_if_microversion_not_supported("2.23")
    @ddt.data((True, "name"), (True, "id"), (False, "name"), (False, "id"))
    @ddt.unpack
    def test_pool_list_with_share_type_filter_with_detail(
            self, detail, share_type_key):
        st = self.create_share_type()
        search_opts = {"share_type": st[share_type_key]}
        kwargs = {'search_opts': search_opts}

        if detail:
            kwargs.update({'detail': True})

        pools = self.admin_client.list_pools(**kwargs)['pools']

        self.assertIsNotNone(pools, 'No pools returned from pools API')
        self.assertNotEmpty(pools)
        for pool in pools:
            pool_keys = list(pool.keys())
            self.assertIn("name", pool_keys)
            self.assertIn("host", pool_keys)
            self.assertIn("backend", pool_keys)
            self.assertIn("pool", pool_keys)
            self.assertIs(detail, "capabilities" in pool_keys)

    @decorators.idempotent_id('31cb2c99-3abf-4dce-8e66-7fd30b168300')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API_WITH_BACKEND)
    @utils.skip_if_microversion_not_supported("2.23")
    @ddt.data((True, "name"), (True, "id"), (False, "name"), (False, "id"))
    @ddt.unpack
    def test_pool_list_with_share_type_filter_with_detail_negative(
            self, detail, share_type_key):
        st_negative = self._create_share_type(negative=True)
        search_opts = {"share_type": st_negative[share_type_key]}

        pools = self.admin_client.list_pools(
            detail=detail, search_opts=search_opts)['pools']

        self.assertEmpty(pools)

    def _wrap_regex_for_exact_match(self, regex):
        return '^%s$' % regex



# Copyright 2015 Mirantis Inc.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from tempest import config
from tempest.lib import decorators
from tempest.lib import exceptions as lib_exc
from testtools import testcase as tc

from manila_tempest_tests.tests.api import base
from manila_tempest_tests import utils

CONF = config.CONF


class ExportLocationsNegativeTest(base.BaseSharesMixedTest):

    @classmethod
    def skip_checks(cls):
        super(ExportLocationsNegativeTest, cls).skip_checks()
        utils.check_skip_if_microversion_not_supported("2.9")

    @classmethod
    def resource_setup(cls):
        super(ExportLocationsNegativeTest, cls).resource_setup()
        # admin_client and different_project_client pertain to isolated
        # projects, admin_member_client is a regular user in admin's project
        cls.admin_client = cls.admin_shares_v2_client
        cls.admin_member_client = (
            cls.admin_project_member_client.shares_v2_client)
        cls.different_project_client = cls.shares_v2_client
        # create share type
        cls.share_type = cls.create_share_type()
        cls.share_type_id = cls.share_type['id']
        # create share
        cls.share = cls.create_share(client=cls.admin_client,
                                     share_type_id=cls.share_type_id)
        cls.share = cls.admin_client.get_share(cls.share['id'])['share']
        cls.share_instances = cls.admin_client.get_instances_of_share(
            cls.share['id'])['share_instances']

    @decorators.idempotent_id('8eac1355-f272-4913-8a49-1a8a9cb086bd')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API_WITH_BACKEND)
    def test_get_inexistent_share_export_location(self):
        self.assertRaises(
            lib_exc.NotFound,
            self.admin_client.get_share_export_location,
            self.share['id'],
            "fake-inexistent-share-instance-id",
        )

    @decorators.idempotent_id('064a18dd-1a00-42f1-84c0-5a3e3b46fb39')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API_WITH_BACKEND)
    def test_get_inexistent_share_instance_export_location(self):
        for share_instance in self.share_instances:
            self.assertRaises(
                lib_exc.NotFound,
                self.admin_client.get_share_instance_export_location,
                share_instance['id'],
                "fake-inexistent-share-instance-id",
            )

    @decorators.idempotent_id('6d0b9d1b-fc87-4b7f-add5-919b0ddcda90')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API_WITH_BACKEND)
    def test_list_share_instance_export_locations_as_member(self):
        for share_instance in self.share_instances:
            self.assertRaises(
                lib_exc.Forbidden,
                self.admin_member_client.list_share_instance_export_locations,
                share_instance['id'])

    @decorators.idempotent_id('abde4357-a26c-4adb-88a6-ece6b0e15b5e')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API_WITH_BACKEND)
    def test_get_share_instance_export_locations_as_member(self):
        for share_instance in self.share_instances:
            export_locations = (
                self.admin_client.list_share_instance_export_locations(
                    share_instance['id'])['export_locations'])
            for el in export_locations:
                self.assertRaises(lib_exc.Forbidden,
                                  (self.admin_member_client.
                                   get_share_instance_export_location),
                                  share_instance['id'], el['id'])

    @decorators.idempotent_id('a3c3d16b-5f62-4089-8f86-efc660592986')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API_WITH_BACKEND)
    def test_list_share_export_locations_by_different_project_user(self):
        self.assertRaises(
            lib_exc.Forbidden,
            self.different_project_client.list_share_export_locations,
            self.share['id'])

    @decorators.idempotent_id('0f6823a5-3929-4025-9cd4-b5198b4384dd')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API_WITH_BACKEND)
    def test_get_share_export_location_by_different_project_user(self):
        export_locations = self.admin_client.list_share_export_locations(
            self.share['id'])['export_locations']

        for export_location in export_locations:
            self.assertRaises(
                lib_exc.Forbidden,
                self.different_project_client.get_share_export_location,
                self.share['id'],
                export_location['id'])


class ExportLocationsAPIOnlyNegativeTest(base.BaseSharesAdminTest):

    @classmethod
    def skip_checks(cls):
        super(ExportLocationsAPIOnlyNegativeTest, cls).skip_checks()
        utils.check_skip_if_microversion_not_supported("2.9")

    @decorators.idempotent_id('4b5b4e89-0c80-4383-b272-62d5e0419d9a')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API)
    def test_get_export_locations_by_nonexistent_share(self):
        self.assertRaises(
            lib_exc.NotFound,
            self.shares_v2_client.list_share_export_locations,
            "fake-inexistent-share-id",
        )

    @decorators.idempotent_id('21ba5111-91a8-4ec3-86dc-689fc2fa90e6')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API)
    def test_get_export_locations_by_nonexistent_share_instance(self):
        self.assertRaises(
            lib_exc.NotFound,
            self.shares_v2_client.list_share_instance_export_locations,
            "fake-inexistent-share-instance-id",
        )



# Copyright 2015 Deutsche Telekom AG
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from tempfile import mkstemp
from urllib.request import urlopen

from oslo_log import log
from tempest.common import waiters
from tempest import config
from tempest.lib.common.utils import data_utils
from tempest.lib.common.utils import test_utils
from tempest.lib import exceptions

from manila_tempest_tests.common import constants
from manila_tempest_tests.common import remote_client
from manila_tempest_tests.common import waiters as share_waiters
from manila_tempest_tests.tests.api import base
from manila_tempest_tests.tests.scenario import manager
from manila_tempest_tests import utils

CONF = config.CONF
LOG = log.getLogger(__name__)


class ShareScenarioTest(manager.NetworkScenarioTest):
    """Provide harness to do Manila scenario tests."""

    credentials = ('admin', 'primary')
    protocol = None
    ip_version = 4

    @property
    def ipv6_enabled(self):
        return self.ip_version == 6

    @classmethod
    def setup_clients(cls):
        super(ShareScenarioTest, cls).setup_clients()

        cls.compute_floating_ips_client = (
            cls.os_primary.compute_floating_ips_client)
        # Manila clients
        cls.shares_client = cls.os_primary.share_v1.SharesClient()
        cls.shares_v2_client = cls.os_primary.share_v2.SharesV2Client()
        cls.shares_admin_client = cls.os_admin.share_v1.SharesClient()
        cls.shares_admin_v2_client = cls.os_admin.share_v2.SharesV2Client()

    @classmethod
    def skip_checks(cls):
        super(ShareScenarioTest, cls).skip_checks()
        if not CONF.service_available.manila:
            raise cls.skipException("Manila support is required")
        if cls.ip_version == 6 and not CONF.share.run_ipv6_tests:
            raise cls.skipException("IPv6 tests are disabled")
        if cls.protocol not in CONF.share.enable_protocols:
            message = "%s tests are disabled" % cls.protocol
            raise cls.skipException(message)

    def setUp(self):
        base.verify_test_has_appropriate_tags(self)
        super(ShareScenarioTest, self).setUp()

        self.image_id = None
        # Setup image and flavor the test instance
        # Support both configured and injected values
        self.floating_ips = {}
        self.storage_network_nic_ips = {}

        if not hasattr(self, 'flavor_ref'):
            self.flavor_ref = CONF.share.client_vm_flavor_ref

        if CONF.share.image_with_share_tools == 'centos':
            self.image_ref = self._create_centos_based_glance_image()
        elif CONF.share.image_with_share_tools:
            images = self.image_client.list_images()["images"]
            for img in images:
                if img["name"] == CONF.share.image_with_share_tools:
                    self.image_id = img['id']
                    break
            if not self.image_id:
                msg = ("Image %s not found. Expecting an image including "
                       "required share tools." %
                       CONF.share.image_with_share_tools)
                raise exceptions.InvalidConfiguration(msg)
        self.ssh_user = CONF.share.image_username
        LOG.debug('Starting test for i:{image_id}, f:{flavor}. '
                  'user: {ssh_user}'.format(image_id=self.image_id,
                                            flavor=self.flavor_ref,
                                            ssh_user=self.ssh_user))

        self.storage_network = (
            self._get_network_by_name_or_id(CONF.share.storage_network)
            if CONF.share.storage_network else None
        )
        self.storage_network_name = (
            self.storage_network['name'] if self.storage_network else None
        )

        # Tests need to be able to ssh into the VM - so we need
        # a security group, and a tenant private network
        self.security_group = self.create_security_group()
        self.network = self.create_network(namestart="manila-share")
        # When not using a "storage network" to connect shares to VMs,
        # we need the subnet to match the IP version we're testing
        subnet_ip_params = {} if self.storage_network else {
            'ip_version': self.ip_version,
            'use_default_subnetpool': self.ipv6_enabled
        }
        self.subnet = self.create_subnet(
            network=self.network,
            namestart="manila-share-sub",
            **subnet_ip_params)
        router = self.get_router()
        self._create_router_interface(subnet_id=self.subnet['id'],
                                      router_id=router['id'])

        if CONF.share.multitenancy_enabled:
            # Skip if DHSS=False
            self.share_network = self.create_share_network()

    def mount_share(self, location, remote_client, target_dir=None):
        raise NotImplementedError

    def allow_access(self, **kwargs):
        raise NotImplementedError

    def unmount_share(self, remote_client, target_dir=None):
        target_dir = target_dir or "/mnt"
        remote_client.exec_command("sudo umount %s" % target_dir)

    def create_share_network(self):
        share_network = self._create_share_network(
            neutron_net_id=self.network['id'],
            neutron_subnet_id=self.subnet['id'],
            name=data_utils.rand_name("sn-name"))
        return share_network

    def boot_instance(self, wait_until="ACTIVE"):
        # In case of multiple instances, use a single keypair to prevent a keys
        # mismatch.
        if not hasattr(self, 'keypair'):
            self.keypair = self.create_keypair()
        security_groups = [{'name': self.security_group['name']}]
        networks = [{'uuid': self.network['id']}]
        if self.storage_network:
            networks.append({'uuid': self.storage_network['id']})

        create_kwargs = {
            'key_name': self.keypair['name'],
            'security_groups': security_groups,
            'wait_until': wait_until,
            'networks': networks,
        }
        instance = self.create_server(
            image_id=self.image_id, flavor=self.flavor_ref, **create_kwargs)
        return instance

    def init_remote_client(self, instance):
        server_ip = None
        if self.ipv6_enabled:
            server_ip = self._get_server_ip(instance, ip_version=6)
        if not server_ip:
            ip_addr = self._get_server_ip(instance)
            # Obtain a floating IP
            floating_ip = self.create_floating_ip(instance, ip_addr=ip_addr)
            self.floating_ips[instance['id']] = floating_ip
            server_ip = floating_ip['floating_ip_address']

            if self.storage_network:
                storage_net_nic = instance['addresses'].get(
                    self.storage_network_name)
                if storage_net_nic:
                    self.storage_network_nic_ips[instance['id']] = (
                        storage_net_nic[0]['addr']
                    )
            # Attach a floating IP
            self.associate_floating_ip(floating_ip, instance)

        self.assertIsNotNone(server_ip)
        # Check ssh
        remote_client = self.get_remote_client(
            server_or_ip=server_ip,
            username=self.ssh_user,
            private_key=self.keypair['private_key'],
            server=instance)

        # NOTE(u_glide): Workaround for bug #1465682
        remote_client = remote_client.ssh_client

        return remote_client

    def validate_ping_to_export_location(self, export, remote_client,
                                         ping_timeout=None):
        timeout = ping_timeout or CONF.validation.ping_timeout

        def ping_to_export_location(export, remote_client):
            ip, version = self.get_ip_and_version_from_export_location(export)
            try:
                remote_client.exec_command(
                    "ping{} -c5 -w1 {}".format(
                        '6' if version == 6 else '', ip))
                return True
            except exceptions.SSHExecCommandFailed:
                return False

        test_utils.call_until_true(ping_to_export_location,
                                   timeout, 1, export=export,
                                   remote_client=remote_client)

    def write_data_to_mounted_share(self, escaped_string, remote_client,
                                    mount_point='/mnt/t1'):
        remote_client.exec_command("echo \"{escaped_string}\" "
                                   "| sudo tee {mount_point} && sudo sync"
                                   .format(escaped_string=escaped_string,
                                           mount_point=mount_point))

    def write_data_to_mounted_share_using_dd(self, remote_client,
                                             output_file,
                                             block_size,
                                             block_count):
        """Writes data to mounted share using dd command

        Example Usage for writing 512Mb to a file on /mnt/
        (remote_client, block_size=1024, block_count=512000,
        output_file='/mnt/512mb_of_zeros', input_file='/dev/zero')

        For more information, refer to the dd man page.

        :param remote_client: An SSH client connection to the Nova instance
        :param block_size: The size of an individual block in bytes
        :param block_count: The number of blocks to write
        :param output_file: Path to the file to be written
        """
        block_count = int(block_count)
        remote_client.exec_command(
            "sudo sh -c \"dd bs={} count={} if={} of={} iflag=fullblock\""
            .format(block_size, block_count, CONF.share.dd_input_file,
                    output_file))

    def read_data_from_mounted_share(self,
                                     remote_client,
                                     mount_point='/mnt/t1'):
        data = remote_client.exec_command("sudo cat {mount_point}"
                                          .format(mount_point=mount_point))
        return data.rstrip()

    def migrate_share(self, share_id, dest_host, status,
                      force_host_assisted=False):
        share = self._migrate_share(
            share_id, dest_host, status, force_host_assisted,
            self.shares_admin_v2_client)
        return share

    def migration_complete(self, share_id, dest_host):
        return self._migration_complete(share_id, dest_host)

    def create_share(self, extra_specs=None, **kwargs):
        kwargs.update({
            'share_protocol': self.protocol,
        })
        if not ('share_type_id' in kwargs or 'snapshot_id' in kwargs):
            default_share_type_id = self.get_share_type(
                extra_specs=extra_specs)['id']
            kwargs.update({'share_type_id': default_share_type_id})
        if CONF.share.multitenancy_enabled:
            kwargs.update({'share_network_id': self.share_network['id']})
        share = self._create_share(**kwargs)
        return share

    def get_remote_client(self, *args, **kwargs):
        if not CONF.share.image_with_share_tools:
            return super(ShareScenarioTest,
                         self).get_remote_client(*args, **kwargs)
        # NOTE(u_glide): We need custom implementation of this method until
        # original implementation depends on CONF.compute.ssh_auth_method
        # option.
        server_or_ip = kwargs['server_or_ip']
        if isinstance(server_or_ip, str):
            ip = server_or_ip
        else:
            addr = server_or_ip['addresses'][
                CONF.validation.network_for_ssh][0]
            ip = addr['addr']

        # NOTE(u_glide): Both options (pkey and password) are required here to
        # support service images without Nova metadata support
        client_params = {
            'username': kwargs['username'],
            'password': CONF.share.image_password,
            'pkey': kwargs.get('private_key'),
        }

        linux_client = remote_client.RemoteClient(ip, **client_params)
        try:
            linux_client.validate_authentication()
        except Exception:
            LOG.exception('Initializing SSH connection to %s failed', ip)
            self.log_console_output()
            raise

        return linux_client

    def allow_access_ip(self, share_id, ip=None, instance=None,
                        access_level="rw", cleanup=True, snapshot=None,
                        client=None):
        client = client or self.shares_v2_client
        if instance and not ip:
            try:
                net_addresses = instance['addresses']
                first_address = list(net_addresses.values())[0][0]
                ip = first_address['addr']
            except Exception:
                LOG.debug("Instance has no valid IP address: %s", instance)
                # In case on an error ip will be still none
                LOG.exception("Instance has no valid IP address. "
                              "Falling back to default")
        if not ip:
            ip = '::/0' if self.ipv6_enabled else '0.0.0.0/0'

        if snapshot:
            self._allow_access_snapshot(snapshot['id'], access_type='ip',
                                        access_to=ip, cleanup=cleanup,
                                        client=client)
        else:
            return self._allow_access(share_id, access_type='ip',
                                      access_level=access_level, access_to=ip,
                                      cleanup=cleanup,
                                      client=client)

    def deny_access(self, share_id, access_rule_id, client=None):
        """Deny share access

        :param share_id: id of the share
        :param access_rule_id: id of the rule that will be deleted
        """
        client = client or self.shares_client
        client.delete_access_rule(share_id, access_rule_id)
        share_waiters.wait_for_resource_status(
            self.shares_v2_client, share_id, "active",
            status_attr='access_rules_status')

    def _provide_access_to_client_identified_by_ip(self, instance, share=None,
                                                   snapshot=None,
                                                   access_level='rw',
                                                   client=None):
        client = client or self.shares_v2_client
        if not CONF.share.multitenancy_enabled:
            if self.ipv6_enabled and not self.storage_network:
                server_ip = self._get_server_ip(instance, ip_version=6)
            else:
                server_ip = (
                    CONF.share.override_ip_for_nfs_access
                    or self.storage_network_nic_ips.get(instance['id'])
                    or self.floating_ips[instance['id']]['floating_ip_address']
                )

            self.assertIsNotNone(server_ip)
            return self.allow_access_ip(
                share['id'], ip=server_ip,
                instance=instance, cleanup=False, snapshot=snapshot,
                access_level=access_level, client=client)
        else:
            return self.allow_access_ip(
                share['id'], instance=instance, cleanup=False,
                snapshot=snapshot, access_level=access_level, client=client)

    def _provide_access_to_client_identified_by_cephx(self, share=None,
                                                      access_rule=None,
                                                      access_level='rw',
                                                      access_to=None,
                                                      remote_client=None,
                                                      locations=None,
                                                      client=None,
                                                      oc_size=20971520):
        """Provide an access to a client identified by cephx authentication

        :param: share: An existing share.
        :param: access_rule: An existing access rule. In case we want to create
                        the configuration files in the instance according to an
                        existing access rule.
        :param: access_level: Share access level; this is not required if
                        "access_rule" is set.
        :param: access_to: Client to provide access to; this is not required if
                        "access_rule" is set.
        :param: remote_client: An SSH client connection to the Nova instance.
        :param: locations: Export locations of shares.
        :param: client: Client object.
        :param: oc_size: Set how many bytes of data will the client cache.
        :return: Share access.

        """
        client = client or self.shares_v2_client
        if not access_rule:
            access_to = access_to or data_utils.rand_name(
                self.__class__.__name__ + '-cephx-id')
            # Check if access is already granted to the client
            access_rules_matching_client = client.list_access_rules(
                share['id'],
                metadata={'metadata': {'access_to': access_to}})['access_list']
            access_rule = (access_rules_matching_client[0] if
                           access_rules_matching_client else None)

            if not access_rule:
                access_rule = self._allow_access(
                    share['id'], access_level=access_level,
                    access_to=access_to, access_type="cephx", cleanup=False,
                    client=client)
                # Set metadata to access rule to be filtered if necessary.
                # This is necessary to prevent granting access to a client who
                # already has.
                client.update_access_metadata(
                    metadata={"access_to": "{}".format(access_to)},
                    access_id=access_rule['id'])
        get_access = client.get_access_rule(access_rule['id'])['access']
        # Set 'access_key' and 'access_to' attributes for being use in mount
        # operation.
        setattr(self, 'access_key', get_access['access_key'])
        setattr(self, 'access_to', get_access['access_to'])

        remote_client.exec_command(
            "sudo crudini --set {access_to}.keyring client.{access_to} key "
            "{access_key}"
            .format(access_to=self.access_to, access_key=self.access_key))
        remote_client.exec_command(
            "sudo crudini --set ceph.conf client \"client quota\" true")
        remote_client.exec_command(
            "sudo crudini --set ceph.conf client \"client oc size\" {}"
            .format(oc_size))
        if not isinstance(locations, list):
            locations = [locations]
        remote_client.exec_command(
            "sudo crudini --set ceph.conf client \"mon host\" {}"
            .format(locations[0].split(':/')[0]))
        return access_rule

    def wait_for_active_instance(self, instance_id):
        waiters.wait_for_server_status(
            self.os_primary.servers_client, instance_id, "ACTIVE")
        return self.os_primary.servers_client.show_server(
            instance_id)["server"]

    def get_share_type(self, extra_specs=None):
        if CONF.share.default_share_type_name:
            return self.shares_client.get_default_share_type()['share_type']
        extra_specs_dict = {
            'driver_handles_share_servers': CONF.share.multitenancy_enabled
        }
        if extra_specs:
            extra_specs_dict.update(extra_specs)
        return self._create_share_type(
            data_utils.rand_name("share_type"),
            extra_specs=extra_specs_dict)['share_type']

    def get_share_export_locations(self, share):
        if utils.is_microversion_lt(CONF.share.max_api_microversion, "2.9"):
            locations = share['export_locations']
        else:
            exports = self.shares_v2_client.list_share_export_locations(
                share['id'])['export_locations']
            locations = [x['path'] for x in exports]
        return locations

    def _get_snapshot_export_locations(self, snapshot):
        exports = (self.shares_v2_client.list_snapshot_export_locations(
            snapshot['id'])['share_snapshot_export_locations'])
        locations = [x['path'] for x in exports]

        return locations

    def _get_server_ip(self, instance, ip_version=4):
        ip_addrs = []
        for network_name, nic_list in instance['addresses'].items():
            if network_name == self.storage_network_name:
                continue
            for nic_data in nic_list:
                if nic_data['version'] == ip_version:
                    ip_addrs.append(nic_data['addr'])
        return ip_addrs[0] if ip_addrs else None

    def _create_share(self, share_protocol=None, size=None, name=None,
                      snapshot_id=None, description=None, metadata=None,
                      share_network_id=None, share_type_id=None,
                      client=None, cleanup=True):
        """Create a share

        :param share_protocol: NFS or CIFS
        :param size: size in GB
        :param name: name of the share (otherwise random)
        :param snapshot_id: snapshot as basis for the share
        :param description: description of the share
        :param metadata: adds additional metadata
        :param share_network_id: id of network to be used
        :param share_type_id: type of the share to be created
        :param client: client object
        :param cleanup: default: True
        :returns: a created share
        """
        client = client or self.shares_client
        description = description or "Tempest's share"
        if not name:
            name = data_utils.rand_name("manila-scenario")
        if CONF.share.multitenancy_enabled:
            share_network_id = (share_network_id or client.share_network_id)
        else:
            share_network_id = None
        metadata = metadata or {}
        kwargs = {
            'share_protocol': share_protocol,
            'size': size or CONF.share.share_size,
            'name': name,
            'snapshot_id': snapshot_id,
            'description': description,
            'metadata': metadata,
            'share_network_id': share_network_id,
            'share_type_id': share_type_id,
        }
        share = self.shares_client.create_share(**kwargs)['share']

        if cleanup:
            self.addCleanup(client.wait_for_resource_deletion,
                            share_id=share['id'])
            self.addCleanup(client.delete_share,
                            share['id'])

        share_waiters.wait_for_resource_status(client, share['id'],
                                               'available')
        return share

    def _create_snapshot(self, share_id, client=None, **kwargs):
        client = client or self.shares_v2_client
        snapshot = client.create_snapshot(share_id, **kwargs)['snapshot']
        self.addCleanup(
            client.wait_for_resource_deletion, snapshot_id=snapshot['id'])
        self.addCleanup(client.delete_snapshot, snapshot['id'])
        share_waiters.wait_for_resource_status(
            client, snapshot["id"], "available", resource_name='snapshot')
        return snapshot

    def _wait_for_share_server_deletion(self, sn_id, client=None):
        """Wait for a share server to be deleted

        :param sn_id: shared network id
        :param client: client object
        """
        client = client or self.shares_admin_client
        servers = client.list_share_servers(
            search_opts={"share_network": sn_id})['share_servers']
        for server in servers:
            client.delete_share_server(server['id'])
        for server in servers:
            client.wait_for_resource_deletion(server_id=server['id'])

    def _create_share_network(self, client=None, **kwargs):
        """Create a share network

        :param client: client object
        :returns: a created share network
        """

        client = client or self.shares_client
        sn = client.create_share_network(**kwargs)['share_network']

        self.addCleanup(client.wait_for_resource_deletion,
                        sn_id=sn['id'])
        self.addCleanup(client.delete_share_network,
                        sn['id'])
        self.addCleanup(self._wait_for_share_server_deletion,
                        sn['id'])
        return sn

    def _allow_access(self, share_id, client=None, access_type="ip",
                      access_level="rw", access_to="0.0.0.0", cleanup=True):
        """Allow share access

        :param share_id: id of the share
        :param client: client object
        :param access_type: "ip", "user" or "cert"
        :param access_level: "rw" or "ro"
        :param access_to
        :returns: access object
        """
        client = client or self.shares_v2_client
        access = client.create_access_rule(share_id, access_type, access_to,
                                           access_level)['access']

        share_waiters.wait_for_resource_status(
            client, share_id, "active", status_attr='access_rules_status')

        if cleanup:
            self.addCleanup(client.delete_access_rule, share_id, access['id'])
        return access

    def _allow_access_snapshot(self, snapshot_id, access_type="ip",
                               access_to="0.0.0.0/0", cleanup=True,
                               client=None):
        """Allow snapshot access

        :param snapshot_id: id of the snapshot
        :param access_type: "ip", "user" or "cert"
        :param access_to
        :param client: shares client, normal/admin
        :returns: access object
        """
        client = client or self.shares_v2_client
        access = client.create_snapshot_access_rule(
            snapshot_id, access_type, access_to)['snapshot_access']

        if cleanup:
            self.addCleanup(client.delete_snapshot_access_rule,
                            snapshot_id, access['id'])

        share_waiters.wait_for_resource_status(
            client, snapshot_id, 'active',
            resource_name='snapshot_access', rule_id=access['id'],
            status_attr='state')

        return access

    def _create_router_interface(self, subnet_id, client=None, router_id=None):
        """Create a router interface

        :param subnet_id: id of the subnet
        :param client: client object
        """
        if not client:
            client = self.routers_client
        if not router_id:
            router_id = self.get_router()['id']
        client.add_router_interface(router_id, subnet_id=subnet_id)
        self.addCleanup(
            client.remove_router_interface, router_id, subnet_id=subnet_id)

    def _migrate_share(self, share_id, dest_host, status, force_host_assisted,
                       client=None):
        client = client or self.shares_admin_v2_client
        client.migrate_share(
            share_id, dest_host, writable=False, preserve_metadata=False,
            nondisruptive=False, preserve_snapshots=False,
            force_host_assisted_migration=force_host_assisted)
        share = share_waiters.wait_for_migration_status(
            client, share_id, dest_host, status)
        return share

    def _migration_complete(self, share_id, dest_host, client=None, **kwargs):
        client = client or self.shares_admin_v2_client
        client.migration_complete(share_id, **kwargs)
        share = share_waiters.wait_for_migration_status(
            client, share_id, dest_host,
            constants.TASK_STATE_MIGRATION_SUCCESS, **kwargs)
        return share

    def _create_share_type(self, name, is_public=True, **kwargs):
        share_type = self.shares_admin_v2_client.create_share_type(name,
                                                                   is_public,
                                                                   **kwargs)
        self.addCleanup(self.shares_admin_v2_client.delete_share_type,
                        share_type['share_type']['id'])
        return share_type

    def _create_centos_based_glance_image(self):
        imagepath = mkstemp(suffix='.qcow2')[1]
        imagefile = open(imagepath, 'wb+')
        image_response = urlopen('http://cloud.centos.org/centos/7/images/' +
                                 'CentOS-7-x86_64-GenericCloud.qcow2')

        LOG.info('Downloading CentOS7 image')
        while True:
            imagecopy = image_response.read(100 * 1024 * 1024)
            if imagecopy == '':
                break
            imagefile.write(imagecopy)

        imagefile.close()

        LOG.info('Creating Glance image using the downloaded image file')
        return self._image_create('centos', 'bare', imagepath, 'qcow2')

    def get_user_export_locations(self, share=None, snapshot=None):
        locations = None
        if share:
            locations = self.get_share_export_locations(share)
        elif snapshot:
            locations = self._get_snapshot_export_locations(snapshot)

        self.assertNotEmpty(locations)
        if self.protocol != 'cephfs':
            locations = self._get_export_locations_according_to_ip_version(
                locations)
            self.assertNotEmpty(locations)

        return locations

    def _get_export_locations_according_to_ip_version(self, all_locations):
        locations = [
            x for x in all_locations
            if self.get_ip_and_version_from_export_location(
                x)[1] == self.ip_version]

        if len(locations) == 0:
            message = ("Configured backend does not support "
                       "ip_version %s" % self.ip_version)
            raise self.skipException(message)
        return locations

    def get_ip_and_version_from_export_location(self, export):
        export = export.replace('[', '').replace(']', '')
        if self.protocol == 'nfs' and ':/' in export:
            ip = export.split(':/')[0]
            version = 6 if ip.count(':') > 1 else 4
        elif self.protocol == 'cifs' and export.startswith(r'\\'):
            ip = export.split('\\')[2]
            version = 6 if (ip.count(':') > 1 or
                            ip.endswith('ipv6-literal.net')) else 4
        else:
            message = ("Protocol %s is not supported" % self.protocol)
            raise self.skipException(message)
        return ip, version


class BaseShareScenarioNFSTest(ShareScenarioTest):
    protocol = "nfs"

    @classmethod
    def skip_checks(cls):
        super(BaseShareScenarioNFSTest, cls).skip_checks()
        if cls.protocol not in CONF.share.enable_ip_rules_for_protocols:
            message = ("%s tests for access rules other than IP are disabled" %
                       cls.protocol)
            raise cls.skipException(message)

    def allow_access(self, access_level='rw', **kwargs):
        snapshot = kwargs.get('snapshot')
        share = kwargs.get('share')
        return self._provide_access_to_client_identified_by_ip(
            share=share, instance=kwargs['instance'],
            access_level=access_level, snapshot=snapshot)

    def mount_share(self, location, ssh_client, target_dir=None):

        self.validate_ping_to_export_location(location, ssh_client)

        target_dir = target_dir or "/mnt"
        ssh_client.exec_command(
            "sudo mount -vt nfs \"%s\" %s" % (location, target_dir)
        )


class BaseShareScenarioCIFSTest(ShareScenarioTest):
    protocol = 'cifs'

    @classmethod
    def skip_checks(cls):
        super(BaseShareScenarioCIFSTest, cls).skip_checks()
        if cls.protocol not in CONF.share.enable_ip_rules_for_protocols:
            message = ("%s tests for access rules other than IP are disabled" %
                       cls.protocol)
            raise cls.skipException(message)

    def allow_access(self, access_level='rw', **kwargs):
        share = kwargs.get('share')
        snapshot = kwargs.get('snapshot')
        return self._provide_access_to_client_identified_by_ip(
            instance=kwargs['instance'],
            share=share,
            snapshot=snapshot,
            access_level=access_level)

    def mount_share(self, location, ssh_client, target_dir=None):

        self.validate_ping_to_export_location(location, ssh_client)

        location = location.replace("\\", "/")
        target_dir = target_dir or "/mnt"
        ssh_client.exec_command(
            "sudo mount.cifs \"%s\" %s -o guest" % (location, target_dir)
        )


class BaseShareScenarioCEPHFSTest(ShareScenarioTest):
    protocol = 'cephfs'

    def allow_access(self, access_level='rw', access_rule=None, **kwargs):
        return self._provide_access_to_client_identified_by_cephx(
            share=kwargs['share'], remote_client=kwargs['remote_client'],
            locations=kwargs['locations'], access_level=access_level,
            access_rule=access_rule)

    def _fuse_client(self, mountpoint, remote_client, target_dir, access_to):
        remote_client.exec_command(
            "sudo ceph-fuse {target_dir} --id={access_to} --conf=ceph.conf "
            "--keyring={access_to}.keyring --client-mountpoint={mountpoint}"
            .format(target_dir=target_dir, access_to=access_to,
                    mountpoint=mountpoint))

    def mount_share(self, location, remote_client, target_dir=None,
                    access_to=None):
        target_dir = target_dir or "/mnt"
        access_to = access_to or self.access_to
        mountpoint = location.split(':')[-1]
        if getattr(self, 'mount_client', None):
            return self._fuse_client(mountpoint, remote_client, target_dir,
                                     access_to=access_to)
        remote_client.exec_command(
            "sudo mount -t ceph {location} {target_dir} -o name={access_to},"
            "secret={access_key}"
            .format(location=location, target_dir=target_dir,
                    access_to=access_to, access_key=self.access_key))

    def unmount_share(self, remote_client, target_dir=None):
        target_dir = target_dir or "/mnt"
        if getattr(self, 'mount_client', None):
            return remote_client.exec_command(
                "sudo fusermount -uz %s" % target_dir)
        super(BaseShareScenarioCEPHFSTest, self).unmount_share(
            remote_client, target_dir=target_dir)



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import time

from oslo_log import log as logging
from tempest import config
from tempest.lib import decorators
from tempest.lib import exceptions
import testtools
from testtools import testcase as tc

from manila_tempest_tests.common import constants
from manila_tempest_tests.common import waiters
from manila_tempest_tests.tests.api import base
from manila_tempest_tests.tests.scenario import manager_share as manager


CONF = config.CONF
LOG = logging.getLogger(__name__)


class ShareShrinkBase(manager.ShareScenarioTest):

    """This test case uses the following flow:

     * Launch an instance
     * Create share (Configured size + 1)
     * Configure RW access to the share
     * Perform ssh to instance
     * Mount share
     * Write data in share (in excess of 1GB)
     * Shrink share to 1GB (fail expected)
     * Delete data from share
     * Shrink share to 1GB
     * Write more than 1GB of data (fail expected)
     * Unmount share
     * Delete share
     * Terminate the instance
    """

    @decorators.idempotent_id('ed0f9c0c-5302-4cc9-9f5d-f7641cc3b83b')
    @tc.attr(base.TAG_POSITIVE, base.TAG_BACKEND)
    @testtools.skipUnless(
        CONF.share.run_shrink_tests, 'Shrink share tests are disabled.')
    def test_create_shrink_and_write(self):
        default_share_size = CONF.share.share_size
        share_size = CONF.share.share_size + 1

        LOG.debug('Step 1 - create instance')
        instance = self.boot_instance(wait_until="BUILD")

        LOG.debug('Step 2 - create share of size {} Gb'.format(share_size))
        share = self.create_share(size=share_size)

        LOG.debug('Step 3 - wait for active instance')
        instance = self.wait_for_active_instance(instance["id"])
        remote_client = self.init_remote_client(instance)

        LOG.debug('Step 4 - grant access')
        location = self.get_user_export_locations(share)[0]
        self.allow_access(share=share, instance=instance,
                          remote_client=remote_client, locations=location)

        LOG.debug('Step 5 - mount')
        self.mount_share(location, remote_client)

        total_blocks = (1024 * default_share_size) / 64
        blocks = total_blocks + 4
        LOG.debug('Step 6 - writing {} * 64MB blocks'.format(blocks))
        self.write_data_to_mounted_share_using_dd(remote_client,
                                                  '/mnt/t1', '64M',
                                                  blocks)
        time.sleep(CONF.share.share_resize_sync_delay)
        ls_result = remote_client.exec_command("sudo ls -lAh /mnt/")
        LOG.debug(ls_result)

        LOG.debug('Step 8 - try update size, shrink and wait')
        self.shares_v2_client.shrink_share(share['id'],
                                           new_size=default_share_size)
        waiters.wait_for_resource_status(
            self.shares_v2_client, share['id'],
            ['shrinking_possible_data_loss_error', 'available'])

        share = self.shares_v2_client.get_share(share["id"])['share']

        if share["status"] == constants.STATUS_AVAILABLE:
            params = {'resource_id': share['id']}
            messages = self.shares_v2_client.list_messages(
                params=params)['messages']
            self.assertIn('009',
                          [message['action_id'] for message in messages])
            self.assertEqual(share_size, int(share["size"]))

        LOG.debug('Step 9 - delete data')
        remote_client.exec_command("sudo rm /mnt/t1")

        ls_result = remote_client.exec_command("sudo ls -lAh /mnt/")
        LOG.debug(ls_result)

        # Deletion of files can be an asynchronous activity on the backend.
        # Thus we need to wait until timeout for the space to be released
        # and repeating the shrink request until success
        LOG.debug('Step 10 - reset and shrink')
        self.share_shrink_retry_until_success(share["id"],
                                              new_size=default_share_size)

        share = self.shares_v2_client.get_share(share["id"])['share']
        self.assertEqual(default_share_size, int(share["size"]))

        LOG.debug('Step 11 - write more data than allocated, should fail')
        overflow_blocks = blocks + CONF.share.additional_overflow_blocks
        self.assertRaises(
            exceptions.SSHExecCommandFailed,
            self.write_data_to_mounted_share_using_dd,
            remote_client, '/mnt/t1', '64M', overflow_blocks)

        LOG.debug('Step 12 - unmount')
        self.unmount_share(remote_client)

    def share_shrink_retry_until_success(self, share_id, new_size,
                                         status_attr='status'):
        """Try share reset, followed by shrink, until timeout"""

        check_interval = CONF.share.build_interval * 2
        share = self.shares_v2_client.get_share(share_id)['share']
        share_current_size = share["size"]
        share_status = share[status_attr]
        start = int(time.time())
        while share_current_size != new_size:
            if (share_status ==
                    constants.STATUS_SHRINKING_POSSIBLE_DATA_LOSS_ERROR):
                self.shares_admin_v2_client.reset_state(
                    share_id, status=constants.STATUS_AVAILABLE)
            elif share_status != constants.STATUS_SHRINKING:
                try:
                    self.shares_v2_client.shrink_share(share_id,
                                                       new_size=new_size)
                except exceptions.BadRequest as e:
                    if ('New size for shrink must be less than current size'
                            in str(e)):
                        break

            time.sleep(check_interval)
            share = self.shares_v2_client.get_share(share_id)['share']
            share_status = share[status_attr]
            share_current_size = share["size"]
            if share_current_size == new_size:
                return
            if int(time.time()) - start >= CONF.share.build_timeout:
                message = ("Share %(share_id)s failed to shrink within the "
                           "required time %(seconds)s." %
                           {"share_id": share["id"],
                            "seconds": CONF.share.build_timeout})
                raise exceptions.TimeoutException(message)


class TestShareShrinkNFS(manager.BaseShareScenarioNFSTest, ShareShrinkBase):
    pass


class TestShareShrinkCIFS(manager.BaseShareScenarioCIFSTest, ShareShrinkBase):
    pass


class TestBaseShareShrinkScenarioCEPHFS(manager.BaseShareScenarioCEPHFSTest,
                                        ShareShrinkBase):
    @decorators.idempotent_id('7fb324ed-7479-4bd9-b022-b3739dee9bcb')
    @tc.attr(base.TAG_POSITIVE, base.TAG_BACKEND)
    def test_create_shrink_and_write_with_ceph_fuse_client(self):
        self.mount_client = 'fuse'
        super(TestBaseShareShrinkScenarioCEPHFS,
              self).test_create_shrink_and_write()


class TestShareShrinkNFSIPv6(TestShareShrinkNFS):
    ip_version = 6


# NOTE(u_glide): this function is required to exclude ShareShrinkBase from
# executed test cases.
# See: https://docs.python.org/3/library/unittest.html#load-tests-protocol
# for details.
def load_tests(loader, tests, _):
    result = []
    for test_case in tests:
        if type(test_case._tests[0]) is ShareShrinkBase:
            continue
        result.append(test_case)
    return loader.suiteClass(result)



# Copyright 2022 Red Hat, Inc.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import abc

from tempest import config
from tempest.lib import decorators
from tempest.lib import exceptions as lib_exc
from testtools import testcase as tc

from manila_tempest_tests.tests.api import base
from manila_tempest_tests.tests.rbac import base as rbac_base

CONF = config.CONF


class ShareRbacSchedulerStatsTests(rbac_base.ShareRbacBaseTests,
                                   metaclass=abc.ABCMeta):

    @classmethod
    def setup_clients(cls):
        super(ShareRbacSchedulerStatsTests, cls).setup_clients()
        cls.persona = getattr(cls, 'os_%s' % cls.credentials[0])
        cls.client = cls.persona.share_v2.SharesV2Client()

    @abc.abstractmethod
    def test_list_storage_pools(self):
        pass


class ProjectAdminTests(ShareRbacSchedulerStatsTests, base.BaseSharesTest):

    credentials = ['project_admin']

    @decorators.idempotent_id('1ec4d0f5-0d60-4bbc-88a4-57fa92f6f62f')
    @tc.attr(base.TAG_POSITIVE, base.TAG_API)
    def test_list_storage_pools(self):
        self.do_request(
            'list_pools', expected_status=200)


class ProjectMemberTests(ShareRbacSchedulerStatsTests, base.BaseSharesTest):

    credentials = ['project_member']

    @decorators.idempotent_id('905aa5ea-eff9-4022-be41-df7a8593809d')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API)
    def test_list_storage_pools(self):
        self.do_request(
            'list_pools', expected_status=lib_exc.Forbidden)


class ProjectReaderTests(ShareRbacSchedulerStatsTests, base.BaseSharesTest):

    credentials = ['project_reader']

    @decorators.idempotent_id('faab12f9-ff51-458d-af47-362d872761e9')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API)
    def test_list_storage_pools(self):
        self.do_request(
            'list_pools', expected_status=lib_exc.Forbidden)



# Copyright 2015 Mirantis Inc.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from tempest.lib import decorators
from tempest.lib import exceptions as lib_exc
from testtools import testcase as tc

from manila_tempest_tests.tests.api import base


class SchedulerStatsNegativeTest(base.BaseSharesTest):

    @decorators.idempotent_id('33db76eb-7501-43f9-8c4f-8e6e35519a4d')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API)
    def test_try_list_pools_with_user(self):
        self.assertRaises(lib_exc.Forbidden,
                          self.shares_client.list_pools)

    @decorators.idempotent_id('f89136cf-34a1-475b-8514-3114f9f159a5')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API)
    def test_try_list_pools_detailed_with_user(self):
        self.assertRaises(lib_exc.Forbidden,
                          self.shares_client.list_pools,
                          detail=True)



# Copyright 2021 Cloudification GmbH
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from tempest import config
from tempest.lib import decorators
from testtools import testcase as tc

from manila_tempest_tests.tests.api import base
from manila_tempest_tests import utils

CONF = config.CONF


class SharesSchedulerHintsTest(base.BaseSharesMixedTest):

    @classmethod
    def skip_checks(cls):
        super(SharesSchedulerHintsTest, cls).skip_checks()
        if not CONF.share.multi_backend:
            raise cls.skipException("Manila multi-backend is disabled.")
        elif len(CONF.share.backend_names) < 2:
            raise cls.skipException("For running multi-backend tests required"
                                    " two names in config. Skipping.")
        elif any(not name for name in CONF.share.backend_names):
            raise cls.skipException("Share backend names can not be empty. "
                                    "Skipping.")
        utils.check_skip_if_microversion_not_supported('2.65')

    @classmethod
    def resource_setup(cls):
        super(SharesSchedulerHintsTest, cls).resource_setup()
        # create share type
        cls.share_type = cls.create_share_type()
        cls.share_type_id = cls.share_type['id']

        # create share
        cls.share_a = cls.create_share(share_type_id=cls.share_type_id)

    @decorators.idempotent_id('f96d5836-bfc9-4c22-888e-3f62d731573c')
    @tc.attr(base.TAG_POSITIVE, base.TAG_API_WITH_BACKEND)
    def test_same_host_scheduler_hint_in_share_creation(self):
        scheduler_hint = {"same_host": "%s" % self.share_a["id"]}

        # create share with metadata
        share_b = self.create_share(share_type_id=self.share_type_id,
                                    scheduler_hints=scheduler_hint,
                                    cleanup_in_class=False)

        # get backend of shares
        share_a = self.admin_shares_v2_client.get_share(
            self.share_a['id'])['share']
        backend_a = share_a['host']
        share_b = self.admin_shares_v2_client.get_share(
            share_b['id'])['share']
        backend_b = share_b['host']

        # verify same backends
        self.assertEqual(backend_a, backend_b)

        # get metadata of share
        metadata_a = self.shares_v2_client.get_metadata(
            self.share_a["id"])['metadata']
        md_a = {"__affinity_same_host": "%s" % share_b["id"]}
        metadata_b = self.shares_v2_client.get_metadata(
            share_b["id"])['metadata']
        md_b = {"__affinity_same_host": "%s" % self.share_a["id"]}

        # verify metadata
        self.assertEqual(md_a, metadata_a)
        self.assertEqual(md_b, metadata_b)

    @decorators.idempotent_id('6569e0c3-43c9-4ee2-84ff-ea7fa8da8110')
    @tc.attr(base.TAG_POSITIVE, base.TAG_API_WITH_BACKEND)
    def test_different_host_scheduler_hint_in_share_creation(self):
        scheduler_hint = {"different_host": "%s" % self.share_a["id"]}

        # create share with metadata
        share_c = self.create_share(share_type_id=self.share_type_id,
                                    scheduler_hints=scheduler_hint,
                                    cleanup_in_class=False)

        # get backend of shares
        share_a = self.admin_shares_v2_client.get_share(
            self.share_a['id'])['share']
        backend_a = share_a['host']
        share_c = self.admin_shares_v2_client.get_share(share_c['id'])['share']
        backend_c = share_c['host']

        # verify different backends
        self.assertNotEqual(backend_a, backend_c)



# Copyright 2022 Red Hat, Inc.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import abc

from tempest import config
from tempest.lib import decorators
from tempest.lib import exceptions as lib_exc
from testtools import testcase as tc

from manila_tempest_tests.common import constants
from manila_tempest_tests.common import waiters
from manila_tempest_tests.tests.api import base
from manila_tempest_tests.tests.rbac import base as rbac_base

CONF = config.CONF


class ShareRbacManageShareTests(rbac_base.ShareRbacBaseTests,
                                metaclass=abc.ABCMeta):

    @classmethod
    def skip_checks(cls):
        super(ShareRbacManageShareTests, cls).skip_checks()
        if not CONF.share.run_manage_unmanage_tests:
            raise cls.skipException('Manage/unmanage tests are disabled.')
        if cls.protocol not in CONF.share.enable_protocols:
            message = "%s tests are disabled" % cls.protocol
            raise cls.skipException(message)

    @classmethod
    def setup_clients(cls):
        super(ShareRbacManageShareTests, cls).setup_clients()
        cls.persona = getattr(cls, 'os_%s' % cls.credentials[0])
        cls.client = cls.persona.share_v2.SharesV2Client()
        cls.admin_shares_v2_client = (
            cls.os_project_admin.share_v2.SharesV2Client())
        cls.alt_project_share_v2_client = (
            cls.os_project_alt_member.share_v2.SharesV2Client())

    @classmethod
    def resource_setup(cls):
        super(ShareRbacManageShareTests, cls).resource_setup()
        cls.share_type = cls.get_share_type()

    def share_manage_preparations(self, share_id, unmanage=True):
        share_info = self.admin_shares_v2_client.get_share(share_id)['share']
        export_path = self.admin_shares_v2_client.list_share_export_locations(
            share_id)['export_locations'][0]
        protocol = share_info['share_proto']
        service_host = share_info['host']

        if unmanage:
            self.admin_shares_v2_client.unmanage_share(share_id)
            self.admin_shares_v2_client.wait_for_resource_deletion(
                share_id=share_id)
        return {
            'export_path': export_path,
            'protocol': protocol,
            'service_host': service_host
        }

    @abc.abstractmethod
    def test_manage_share(self):
        pass

    @abc.abstractmethod
    def test_unmanage_share(self):
        pass


class TestProjectAdminTestsNFS(ShareRbacManageShareTests, base.BaseSharesTest):

    credentials = ['project_admin', 'project_alt_member']
    protocol = 'nfs'

    @classmethod
    def setup_clients(cls):
        super(TestProjectAdminTestsNFS, cls).setup_clients()
        project_member = cls.setup_user_client(
            cls.persona, project_id=cls.persona.credentials.project_id)
        cls.share_member_client = project_member.share_v2.SharesV2Client()

    @decorators.idempotent_id('8a21f805-2d45-4b0c-8ec5-3f45337bbf66')
    @tc.attr(base.TAG_POSITIVE, base.TAG_API_WITH_BACKEND)
    def test_manage_share(self):
        share = self.create_share(
            self.share_member_client, self.share_type['id'])
        share_data = self.share_manage_preparations(share['id'])
        self.do_request(
            'manage_share', expected_status=200,
            share_type_id=self.share_type['id'], **share_data)

        alt_share = self.create_share(
            self.alt_project_share_v2_client, self.share_type['id'])
        alt_share_data = self.share_manage_preparations(alt_share['id'])
        self.do_request(
            'manage_share', expected_status=200,
            share_type_id=self.share_type['id'], **alt_share_data)

    @decorators.idempotent_id('be5b836d-d6cc-40a5-acf4-e5f249035383')
    @tc.attr(base.TAG_POSITIVE, base.TAG_API_WITH_BACKEND)
    def test_unmanage_share(self):
        share = self.create_share(
            self.share_member_client, self.share_type['id'])
        share_data = self.share_manage_preparations(
            share['id'], unmanage=False)
        self.do_request(
            'unmanage_share', expected_status=202, share_id=share['id'])
        self.shares_v2_client.wait_for_resource_deletion(share_id=share['id'])

        # Unmanaged share operation removes the share from the management of
        # the shared file systems service without deleting the share.
        # In order to be able to delete the share we need to manage it again,
        # otherwise, it would leave some allocated space.
        managed_share = self.client.manage_share(
            share_type_id=self.share_type['id'], **share_data)['share']
        waiters.wait_for_resource_status(
            self.client, managed_share['id'], constants.STATUS_AVAILABLE)

        alt_share = self.create_share(
            self.alt_project_share_v2_client, self.share_type['id'])
        share_data = self.share_manage_preparations(
            alt_share['id'], unmanage=False)
        self.do_request(
            'unmanage_share', expected_status=202, share_id=alt_share['id'])
        self.shares_v2_client.wait_for_resource_deletion(
            share_id=alt_share['id'])

        alt_managed_share = self.client.manage_share(
            share_type_id=self.share_type['id'], **share_data)['share']
        waiters.wait_for_resource_status(
            self.client, alt_managed_share['id'], constants.STATUS_AVAILABLE)


class TestProjectMemberTestsNFS(ShareRbacManageShareTests,
                                base.BaseSharesTest):

    credentials = ['project_member', 'project_admin', 'project_alt_member']
    protocol = 'nfs'

    @decorators.idempotent_id('46f884b2-531d-41c0-8455-8874629b3ea3')
    @tc.attr(base.TAG_POSITIVE, base.TAG_API_WITH_BACKEND)
    def test_manage_share(self):
        share_client = getattr(self, 'share_member_client', self.client)
        share = self.create_share(share_client, self.share_type['id'])
        share_data = self.share_manage_preparations(
            share['id'], unmanage=False)
        self.do_request(
            'manage_share', expected_status=lib_exc.Forbidden,
            share_type_id=self.share_type['id'], **share_data)

        alt_share = self.create_share(
            self.alt_project_share_v2_client, self.share_type['id'])
        alt_share_data = self.share_manage_preparations(
            alt_share['id'], unmanage=False)
        self.do_request(
            'manage_share', expected_status=lib_exc.Forbidden,
            share_type_id=self.share_type['id'], **alt_share_data)

    @decorators.idempotent_id('9dc2b1a5-8195-46b8-a28a-9710be352f18')
    @tc.attr(base.TAG_POSITIVE, base.TAG_API_WITH_BACKEND)
    def test_unmanage_share(self):
        share_client = getattr(self, 'share_member_client', self.client)
        share = self.create_share(share_client, self.share_type['id'])
        self.do_request(
            'unmanage_share', expected_status=lib_exc.Forbidden,
            share_id=share['id'])

        alt_share = self.create_share(
            self.alt_project_share_v2_client, self.share_type['id'])
        self.do_request(
            'unmanage_share', expected_status=lib_exc.Forbidden,
            share_id=alt_share['id'])


class TestProjectReaderTestsNFS(TestProjectMemberTestsNFS):
    """Test suite for basic share operations by reader user

    In order to test certain share operations we must create a share resource
    for this. Since reader user is limited in resources creation, we are forced
    to use admin credentials, so we can test other share operations.
    In this class we use admin user to create a member user within reader
    project. That way we can perform a reader actions on this resource.
    """

    credentials = ['project_reader', 'project_admin', 'project_alt_member']

    @classmethod
    def setup_clients(cls):
        super(TestProjectReaderTestsNFS, cls).setup_clients()
        project_member = cls.setup_user_client(
            cls.os_project_admin,
            project_id=cls.persona.credentials.project_id)
        cls.share_member_client = project_member.share_v2.SharesV2Client()

    @decorators.idempotent_id('cec85349-b7e3-440e-bbbc-3bb5999b119a')
    @tc.attr(base.TAG_POSITIVE, base.TAG_API_WITH_BACKEND)
    def test_manage_share(self):
        super(TestProjectReaderTestsNFS, self).test_manage_share()

    @decorators.idempotent_id('a524620c-90b6-496c-8418-c469e711a607')
    @tc.attr(base.TAG_POSITIVE, base.TAG_API_WITH_BACKEND)
    def test_unmanage_share(self):
        super(TestProjectReaderTestsNFS, self).test_unmanage_share()


class TestProjectAdminTestsCEPHFS(TestProjectAdminTestsNFS):
    protocol = 'cephfs'


class TestProjectMemberTestsCEPHFS(TestProjectMemberTestsNFS):
    protocol = 'cephfs'


class TestProjectReaderTestsCEPHFS(TestProjectReaderTestsNFS):
    protocol = 'cephfs'


class TestProjectAdminTestsCIFS(TestProjectAdminTestsNFS):
    protocol = 'cifs'


class TestProjectMemberTestsCIFS(TestProjectMemberTestsNFS):
    protocol = 'cifs'


class TestProjectReaderTestsCIFS(TestProjectReaderTestsNFS):
    protocol = 'cifs'



# Copyright 2020 NetApp Inc.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from collections import defaultdict

from tempest import config
from tempest.lib import decorators
from testtools import testcase as tc

from manila_tempest_tests.tests.api import base
from manila_tempest_tests import utils

CONF = config.CONF


class SharesFromSnapshotAcrossPools(base.BaseSharesMixedTest):
    """Test class for share creation from a snapshot across pools."""

    @classmethod
    def resource_setup(cls):
        super(SharesFromSnapshotAcrossPools, cls).resource_setup()
        # create share_type
        extra_specs = {"create_share_from_snapshot_support": True,
                       "snapshot_support": True}
        cls.share_type = cls.create_share_type(extra_specs=extra_specs)
        cls.share_type_id = cls.share_type['id']
        cls.admin_client = cls.admin_shares_v2_client
        cls.pools = cls.get_pools_matching_share_type(cls.share_type,
                                                      client=cls.admin_client)
        if len(cls.pools) < 2:
            msg = ("Could not find the necessary pools. At least two "
                   "compatibles pools are needed to run the tests to create "
                   "share from snapshot across pools.")
            raise cls.skipException(msg)

        # Availability zones grouped by 'replication_domain'
        cls.rep_domain_azs = defaultdict(set)
        for pool in cls.pools:
            backend = pool['name'].split("#")[0]
            rep_domain = pool['capabilities'].get('replication_domain')

            if rep_domain is not None:
                # Update pools with the availability zone
                pool['availability_zone'] = (
                    cls.get_availability_zones(backends=[backend])[0])
                cls.rep_domain_azs[rep_domain].add(pool['availability_zone'])

    @classmethod
    def skip_checks(cls):
        super(SharesFromSnapshotAcrossPools, cls).skip_checks()
        if not CONF.share.capability_create_share_from_snapshot_support:
            raise cls.skipException(
                'Create share from snapshot tests are disabled.')
        if (not CONF.share
                .run_create_share_from_snapshot_in_another_pool_or_az_tests):
            raise cls.skipException(
                'Create share from snapshot in another pool or az tests are '
                'disabled.')
        utils.check_skip_if_microversion_not_supported("2.54")

    @decorators.idempotent_id('6f1fa7d0-94f2-4373-8730-b0986781cc88')
    @tc.attr(base.TAG_POSITIVE, base.TAG_BACKEND)
    def test_create_share_from_snapshot_across_pools_within_backend(self):
        backends = [pool['backend'] for pool in self.pools]
        duplicated_backend_names = [x for n, x in enumerate(backends)
                                    if x in backends[:n]]
        if not duplicated_backend_names:
            msg = ("Could not find the necessary pools. At least two pools in"
                   " the same backend are needed to run the tests to create"
                   " share from snapshot in another pool in the same backend.")
            raise self.skipException(msg)

        # This filter will return the pool_names of the first duplicated
        # backend
        pool_names = [x['pool'] for x in filter(
            lambda x: x['backend'] == duplicated_backend_names[0], self.pools)]

        # Creating share type setting up the pool_name and backend_name
        extra_specs = {"pool_name": pool_names[0]}
        self.admin_client.update_share_type_extra_specs(
            self.share_type['id'], extra_specs)
        share_type_a_get = self.admin_client.get_share_type(
            self.share_type['id'])

        self.addCleanup(
            self.admin_shares_v2_client.delete_share_type_extra_spec,
            self.share_type['id'], 'pool_name')

        # Create source share
        share_a = self.create_share(
            share_type_id=share_type_a_get["share_type"]["id"])

        # Retrieving the share using admin client because the shares's host
        # field is necessary to do the assert
        share_get_a = self.admin_client.get_share(share_a["id"])['share']

        # Create snapshot from source share
        snap = self.create_snapshot_wait_for_active(share_get_a["id"])

        # There's really no other way of deterministically ensuring a snapshot
        # can be cloned in a different pool, because the scheduler will ensure
        # it finds the best pool with knowledge that make senses at that point
        # in time. Force the creation in another pool using the same share type
        self.admin_client.update_share_type_extra_spec(
            self.share_type['id'], "pool_name", pool_names[1])

        # Create share from snapshot another pool
        share_b = self.create_share(snapshot_id=snap["id"])

        # Retrieving the share using admin client because the shares's host
        # field is necessary to do the assert
        share_get_b = self.admin_client.get_share(share_b['id'])['share']

        # Verify share created from snapshot
        msg = ("Expected snapshot_id %s as "
               "source of share %s" % (snap["id"], share_get_b["snapshot_id"]))
        self.assertEqual(share_get_b["snapshot_id"], snap["id"], msg)

        # Verify different pools
        pool_name_a = share_get_a["host"].split("#")[1]
        pool_name_b = share_get_b["host"].split("#")[1]
        msg = ("The snapshot clone share was created on the same pool as the"
               " source share %s" % pool_name_a)
        self.assertNotEqual(pool_name_a, pool_name_b, msg)

    @decorators.idempotent_id('46f33338-d70d-4c9a-ae27-3dd5b7bd1531')
    @tc.attr(base.TAG_POSITIVE, base.TAG_BACKEND)
    def test_share_from_snapshot_across_azs(self):
        azs = next((self.rep_domain_azs[rep] for rep in self.rep_domain_azs if
                    len(self.rep_domain_azs[rep]) > 1), None)
        if azs is None:
            msg = ("Could not find the necessary azs. At least two azs "
                   "are needed to run the test to create share from snapshot "
                   "across azs.")
            raise self.skipException(msg)
        azs = list(azs)
        share_a = self.create_share(share_type_id=self.share_type_id,
                                    availability_zone=azs[0])

        # Create snapshot
        snap = self.create_snapshot_wait_for_active(share_a["id"])

        # Create share from snapshot
        share_b = self.create_share(availability_zone=azs[1],
                                    snapshot_id=snap["id"])

        # Verify share created from snapshot
        msg = ("Expected snapshot_id %s as "
               "source of share: %s" % (snap["id"], share_b["snapshot_id"]))
        self.assertEqual(share_b["snapshot_id"], snap["id"], msg)

        # Verify different azs
        msg = ("The snapshot clone share was created on the same AZ as the"
               " source share %s" % share_a["availability_zone"])
        self.assertNotEqual(share_b["availability_zone"],
                            share_a["availability_zone"],
                            msg)



# Copyright 2014 Mirantis Inc.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from oslo_log import log
from tempest import config
from tempest.lib import decorators
from tempest.lib import exceptions as lib_exc
import testtools
from testtools import testcase as tc

from manila_tempest_tests.tests.api import base
from manila_tempest_tests import utils

CONF = config.CONF
LOG = log.getLogger(__name__)


class SecServicesMappingNegativeTest(base.BaseSharesMixedTest):

    @classmethod
    def resource_setup(cls):
        super(SecServicesMappingNegativeTest, cls).resource_setup()
        cls.sn = cls.create_share_network(cleanup_in_class=True,
                                          add_security_services=False)
        cls.share_net_info = (
            utils.share_network_get_default_subnet(cls.sn)
            if utils.share_network_subnets_are_supported() else cls.sn)
        cls.ss = cls.create_security_service(cleanup_in_class=True)
        cls.cl = cls.shares_client
        # create share type
        cls.share_type = cls.create_share_type()
        cls.share_type_id = cls.share_type['id']

    @decorators.idempotent_id('e3d17444-8ed4-445e-bc65-c748dbc5d21f')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API)
    def test_add_sec_service_twice_to_share_network(self):
        self.cl.add_sec_service_to_share_network(self.sn["id"], self.ss["id"])
        self.assertRaises(lib_exc.Conflict,
                          self.cl.add_sec_service_to_share_network,
                          self.sn["id"], self.ss["id"])

    @decorators.idempotent_id('3f7af51f-3afa-495c-94b7-e9d29f06cf1d')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API)
    def test_add_nonexistant_sec_service_to_share_network(self):
        self.assertRaises(lib_exc.NotFound,
                          self.cl.add_sec_service_to_share_network,
                          self.sn["id"], "wrong_ss_id")

    @decorators.idempotent_id('85dd5693-a89c-4d05-9416-0e11fbba23f5')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API)
    def test_add_empty_sec_service_id_to_share_network(self):
        self.assertRaises(lib_exc.NotFound,
                          self.cl.add_sec_service_to_share_network,
                          self.sn["id"], "")

    @decorators.idempotent_id('d9af5086-ace9-4be3-8119-e765699c0c91')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API)
    def test_add_sec_service_to_nonexistant_share_network(self):
        self.assertRaises(lib_exc.NotFound,
                          self.cl.add_sec_service_to_share_network,
                          "wrong_sn_id", self.ss["id"])

    @decorators.idempotent_id('7272426d-ab58-4efb-a490-0c78c07fa7fe')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API)
    def test_add_sec_service_to_share_network_with_empty_id(self):
        self.assertRaises(lib_exc.NotFound,
                          self.cl.add_sec_service_to_share_network,
                          "", self.ss["id"])

    @decorators.idempotent_id('f87aefa6-9681-477d-a118-603883849f4f')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API)
    def test_list_sec_services_for_nonexistant_share_network(self):
        self.assertRaises(lib_exc.NotFound,
                          self.cl.list_sec_services_for_share_network,
                          "wrong_id")

    @decorators.idempotent_id('7f8d7527-2d62-478a-ab19-213156777612')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API)
    def test_delete_nonexistant_sec_service_from_share_network(self):
        self.assertRaises(lib_exc.NotFound,
                          self.cl.remove_sec_service_from_share_network,
                          self.sn["id"], "wrong_id")

    @decorators.idempotent_id('be1c9c79-efa1-471e-920b-da4733ad383e')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API)
    def test_delete_sec_service_from_nonexistant_share_network(self):
        self.assertRaises(lib_exc.NotFound,
                          self.cl.remove_sec_service_from_share_network,
                          "wrong_id", self.ss["id"])

    @decorators.idempotent_id('c7c2f66f-81f8-4984-b807-2b9520105a33')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API)
    def test_delete_nonexistant_ss_from_nonexistant_sn(self):
        self.assertRaises(lib_exc.NotFound,
                          self.cl.remove_sec_service_from_share_network,
                          "wrong_id", "wrong_id")

    @decorators.idempotent_id('eb66a8f7-b549-4cf1-8719-30844fb151b6')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API_WITH_BACKEND)
    @testtools.skipIf(
        not CONF.share.multitenancy_enabled, "Only for multitenancy.")
    def test_delete_ss_from_sn_used_by_share_server(self):
        sn = self.shares_client.get_share_network(
            self.shares_client.share_network_id)['share_network']
        fresh_sn = self.create_share_network(
            add_security_services=False,
            neutron_net_id=sn["neutron_net_id"],
            neutron_subnet_id=sn["neutron_subnet_id"])

        self.shares_client.add_sec_service_to_share_network(
            fresh_sn["id"], self.ss["id"])

        # Security service with fake data is used, so if we use backend driver
        # that fails on wrong data, we expect error here.
        # We require any share that uses our share-network.
        try:
            self.create_share(share_type_id=self.share_type_id,
                              share_network_id=fresh_sn["id"],
                              cleanup_in_class=False)
        except Exception as e:
            # we do wait for either 'error' or 'available' status because
            # it is the only available statuses for proper deletion.
            LOG.warning("Caught exception. It is expected in case backend "
                        "fails having security-service with improper data "
                        "that leads to share-server creation error. "
                        "%s", str(e))

        self.assertRaises(lib_exc.Forbidden,
                          self.cl.remove_sec_service_from_share_network,
                          fresh_sn["id"],
                          self.ss["id"])

    @decorators.idempotent_id('6a15c8ff-eba3-40e5-8fa1-6eab52338672')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API)
    def test_try_map_two_ss_with_same_type_to_sn(self):
        # create share network
        data = utils.generate_share_network_data()

        sn = self.create_share_network(client=self.cl,
                                       add_security_services=False, **data)
        self.assertDictContainsSubset(data, sn)

        # create security services with same type
        security_services = []
        for i in range(2):
            data = utils.generate_security_service_data()
            ss = self.create_security_service(client=self.cl, **data)
            self.assertDictContainsSubset(data, ss)
            security_services.insert(i, ss)

        # Add security service to share network
        self.cl.add_sec_service_to_share_network(
            sn["id"], security_services[0]["id"])

        # Try to add security service with same type
        self.assertRaises(lib_exc.Conflict,
                          self.cl.add_sec_service_to_share_network,
                          sn["id"], security_services[1]["id"])

    @decorators.idempotent_id('d422a15a-1f4c-4531-a092-9216b90c4179')
    @tc.attr(base.TAG_NEGATIVE, base.TAG_API)
    def test_try_delete_ss_that_assigned_to_sn(self):
        # create share network
        data = utils.generate_share_network_data()

        sn = self.create_share_network(client=self.cl,
                                       add_security_services=False, **data)
        self.assertDictContainsSubset(data, sn)

        # create security service
        data = utils.generate_security_service_data()

        ss = self.create_security_service(client=self.cl, **data)
        self.assertDictContainsSubset(data, ss)

        # Add security service to share network
        self.cl.add_sec_service_to_share_network(sn["id"], ss["id"])

        # Try delete ss, that has been assigned to some sn
        self.assertRaises(lib_exc.Forbidden,
                          self.cl.delete_security_service,
                          ss["id"], )

        # remove seurity service from share-network
        self.cl.remove_sec_service_from_share_network(sn["id"], ss["id"])



# Copyright 2015 Andrew Kerr
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import ddt
from tempest.lib import decorators
from testtools import testcase as tc

from manila_tempest_tests.tests.api import base
from manila_tempest_tests import utils


@ddt.ddt
class ShareInstancesTest(base.BaseSharesAdminTest):

    @classmethod
    def resource_setup(cls):
        super(ShareInstancesTest, cls).resource_setup()
        # create share type
        cls.share_type = cls.create_share_type()
        cls.share_type_id = cls.share_type['id']
        # create share
        cls.share = cls.create_share(share_type_id=cls.share_type_id)

    @decorators.idempotent_id('b5211759-17ab-4806-a58c-c65bc8a06087')
    @tc.attr(base.TAG_POSITIVE, base.TAG_API_WITH_BACKEND)
    def test_get_instances_of_share_v2_3(self):
        """Test that we get only the 1 share instance back for the share."""
        share_instances = self.shares_v2_client.get_instances_of_share(
            self.share['id'], version='2.3'
        )['share_instances']

        self.assertEqual(1, len(share_instances),
                         'Too many share instances found; expected 1, '
                         'found %s' % len(share_instances))

        si = share_instances[0]
        self.assertEqual(self.share['id'], si['share_id'],
                         'Share instance %s has incorrect share id value; '
                         'expected %s, got %s.' % (si['id'],
                                                   self.share['id'],
                                                   si['share_id']))

    @decorators.idempotent_id('b2d8ccba-5973-4e72-b3df-52555ce40b4f')
    @tc.attr(base.TAG_POSITIVE, base.TAG_API_WITH_BACKEND)
    def test_list_share_instances_v2_3(self):
        """Test that we list the share instance back."""
        share_instances = self.shares_v2_client.list_share_instances(
            version='2.3'
        )['share_instances']

        share_ids = [si['share_id'] for si in share_instances]

        msg = 'Share instance for share %s was not found.' % self.share['id']
        self.assertIn(self.share['id'], share_ids, msg)

    @decorators.idempotent_id('d4b2544a-96ce-41ed-8b8e-e46ceef91462')
    @tc.attr(base.TAG_POSITIVE, base.TAG_API_WITH_BACKEND)
    @ddt.data('2.3', '2.9', '2.10', '2.30', '2.54')
    def test_get_share_instance(self, version):
        """Test that we get the proper keys back for the instance."""
        utils.check_skip_if_microversion_not_supported(version)

        share_instances = self.shares_v2_client.get_instances_of_share(
            self.share['id'], version=version,
        )['share_instances']

        si = self.shares_v2_client.get_share_instance(
            share_instances[0]['id'], version=version)['share_instance']

        expected_keys = [
            'host', 'share_id', 'id', 'share_network_id', 'status',
            'availability_zone', 'share_server_id', 'created_at',
        ]
        if utils.is_microversion_lt(version, '2.9'):
            expected_keys.extend(["export_location", "export_locations"])
        if utils.is_microversion_ge(version, '2.10'):
            expected_keys.append("access_rules_status")
        if utils.is_microversion_ge(version, '2.11'):
            expected_keys.append("replica_state")
        if utils.is_microversion_ge(version, '2.22'):
            expected_keys.append("share_type_id")
        if utils.is_microversion_ge(version, '2.30'):
            expected_keys.append("cast_rules_to_readonly")
        if utils.is_microversion_ge(version, '2.54'):
            expected_keys.append("progress")
        if utils.is_microversion_ge(version, '2.71'):
            expected_keys.append("updated_at")
        expected_keys = sorted(expected_keys)
        actual_keys = sorted(si.keys())
        self.assertEqual(expected_keys, actual_keys,
                         'Share instance %s returned incorrect keys; '
                         'expected %s, got %s.' % (
                             si['id'], expected_keys, actual_keys))

    @ddt.data('path', 'id')
    @decorators.idempotent_id('c27b415d-341c-42f0-a269-2c94f69fbee1')
    @tc.attr(base.TAG_POSITIVE, base.TAG_API_WITH_BACKEND)
    @utils.skip_if_microversion_not_supported("2.35")
    def test_list_share_instances_with_export_location_path_and_id(
            self, export_location_type):
        share_instances_except = (
            self.shares_v2_client.get_instances_of_share(
                self.share['id']))['share_instances']
        export_locations = (
            self.shares_v2_client.list_share_instance_export_locations(
                share_instances_except[0]['id']))['export_locations']

        filters = {
            'export_location_' + export_location_type:
                export_locations[0][export_location_type],
        }
        share_instances = self.shares_v2_client.list_share_instances(
            params=filters)['share_instances']

        self.assertEqual(1, len(share_instances))
        self.assertEqual(share_instances_except[0]['id'],
                         share_instances[0]['id'])
